module mod_Markov
!  Purpose: 
!	This module containts subroutines that computes
!	approximated Markov chain from AR(1) process.
!
!  Record of revisions:
!	   Date      Programmer   Description of change
!   ==========   ==========   ======================
!   04/05/2007   T. Yamada    Collect subroutines
	implicit none
	contains

	subroutine Tauchen1986(ng,rho,mu,std_eps,std_z,m,tran,probst,node,value)
	!  Purpose:
	!	Approximate the AR(1) process by finite Markov chain.
	!
	!	z(t+1) = (1-rho)*mu + rho*z(t) + epsilon(t)
	!
	!	epsilon(t) is a Gaussian white noise, and it's standard deviation is std_eps.
	!	Usually, this approach works well in the case of N>=9.
	!
	!	The algorithm is based on Tauchen (1986,Economics Letters).
	!
	!  Input:
	!	ng          :the number of approximated nodes
	!	rho         :AR(1) coefficient
	!	mu          :mu
	!	std_eps     :standard deviation of epsilon
	!	std_z       :standard deviation of z(t+1)
	!	m           :width of state space (Tauchen uses m=3)
	!	            :zmax=m*std_eps, zmin=-m*std_eps.
	!	
	!  Output:
	!	tran        :transition matrix
	!	probst      :ergodic distribution
	!	node        :discretized z in [zmin,zmax]
	!	value       :exponent of node with mean one.
	!
	!	External File: mod_generate_grid,nrtype,IMSL
	!
	!  Record of revisions:
	!	   Date      Programmer   Description of change
	!   ==========   ==========   =====================
	!	07/15/2003   T. Yamada    Original
	!	04/05/2007   T. Yamada    Use ANORDF
		use nrtype,only:dp
		use mod_generate_grid,only:grid_uniformly
		use ANORDF_INT
		implicit none

		!****** input ******
		integer,intent(in) :: ng
		real(dp),intent(in) :: m
		real(dp),intent(in) :: rho,mu
		real(dp),intent(inout) :: std_eps,std_z

		!****** output ******
		real(dp),dimension(ng,ng),intent(out) :: tran
		real(dp),dimension(ng),intent(out) :: probst,node,value

		!****** local variables ******
		integer :: i,j
		real(dp) :: check,std_z_test,zmax,zmin,inc
		real(dp) :: cdf_norm,cdf_norm1,cdf_norm2,z_agg
		real(dp),dimension(ng) :: probst1,z_exp
		real(dp),dimension(ng,ng) :: trans
		!-----------------------------

		!/* check consistency of the standard deviation of z(t+1) and epsilon */
		if (std_eps == 0.0) then
			std_eps  = sqrt(1.0-rho**2)*std_z
		elseif (std_z == 0.0) then
			std_z = sqrt(std_eps**2/(1.0-rho**2))
		elseif (std_eps/=0.0 .and. std_z/=0.0) then
			std_z_test = sqrt(std_eps**2/(1.0-rho**2))
			check      = abs((std_z_test/std_z)-1.0)
			if (check > 0.01) then
				write(*,*) "Warning in Tauchen: two standard deviations are different."
				pause
			end if
		else
			write(*,*) "Warning in Tauchen1986: Both Standard Deviations are Zero."
			pause
		end if

		!/* equally spaced nodes */
		zmax =    m*std_z
		zmin = -1.0*zmax
		call grid_uniformly(zmin,zmax,ng,node)
		inc  = (zmax-zmin) / real(ng-1)

		!/* transition marrix */
		do i = 1,ng
			do j = 1,ng
				if (j == 1) then
					cdf_norm  = ANORDF((node(1)-(1.0-rho)*mu-rho*node(i)+inc/2.0)/std_eps)
					tran(i,j) = cdf_norm
				elseif (j == ng) then
					cdf_norm  = ANORDF((node(ng)-(1.0-rho)*mu-rho*node(i)-inc/2.0)/std_eps)
					tran(i,j) = 1.0 - cdf_norm
				else
					cdf_norm1 = ANORDF((node(j)-(1.0-rho)*mu-rho*node(i)+inc/2.0)/std_eps)
					cdf_norm2 = ANORDF((node(j)-(1.0-rho)*mu-rho*node(i)-inc/2.0)/std_eps)
					tran(i,j) = cdf_norm1 - cdf_norm2
				end if
			end do
		end do

		!/* stationary distribution */
		trans  = transpose(tran)
		probst = 1.0/real(ng)
		check  = 1.0
		do i = 1,10000
			if (check < 1.0e-008) exit
			probst1 = matmul(trans,probst)
			check   = maxval(abs(probst1-probst))
			probst  = probst1
		end do
		check = abs(sum(probst)-1.0)
		if (check >= 1.0e-002) then
			write(*,*) "Warning in Tauchen: the distribution is NOT probability measure"
		end if

		!/* normalize expected value to be one */
		z_exp = exp(node)
		z_agg = dot_product(z_exp,probst)
		value = z_exp / z_agg

	end subroutine Tauchen1986


	subroutine TauchenHussey(ng,rho,mu,std_eps,sigma,tran,probst,node,value)
	!  Purpose:
	!	Approximate the AR(1) process by finite Markov chain.
	!
	!	z(t+1) = (1-rho)*mu + rho*z(t) + epsilon(t)
	!
	!	epsilon(t) is a Gaussian white noise, and it's standard deviation is std_eps.
	!	Usually, this approach works well in the case of N>=9.
	!
	!	The algorithm is based on Tauchen and Hussey (1991,Econometrica).
	!
	!  Input:
	!	ng          :the number of approximated nodes
	!	rho         :AR(1) coefficient
	!	mu          :mu
	!	std_eps     :standard deviation of epsilon
	!	sigma       :std. dev. for Gaussian quadrature
	!
	!  Output:
	!	tran        :transition matrix
	!	probst      :ergodic distribution
	!	node        :discretized z in [zmin,zmax]
	!	value       :exponent of node with mean one.
	!
	!	External File: nrtype,IMSL
	!
	!  Record of revisions:
	!	   Date      Programmer   Description of change
	!   ==========   ==========   =====================
	!	04/05/2007   T. Yamada    Original
		use nrtype,only:dp ,pi
		use GQRUL_INT
		implicit none

		!****** input ******
		integer,intent(in) :: ng
		real(dp),intent(in) :: rho,mu
		real(dp),intent(in) :: std_eps,sigma

		!****** output ******
		real(dp),dimension(ng,ng),intent(out) :: tran
		real(dp),dimension(ng),intent(out) :: probst,node,value

		!****** local variables ******
		integer :: i,j
		real(dp) :: check,inc
		real(dp) :: z_prime,z_agg
		real(dp),dimension(ng) :: qx,qw,probst1,z_exp
		real(dp),dimension(ng,ng) :: trans
		!-----------------------------

		!/* Gauss-Hermite quadrature node */
		call GQRUL(ng,qx,qw,IWEIGH=4)
		do i = 1,ng
			node(i) = sqrt(2.0)*sigma*qx(i) + mu
		end do

		!/* transition marrix */
		do i = 1,ng
			do j = 1,ng
				z_prime   = (1.0-rho)*mu + rho*node(i)
				tran(i,j) = qw(j) * norm_pdf(node(j),z_prime,std_eps**2) / norm_pdf(node(j),mu,sigma**2)
			end do
		end do
		do i = 1,ng
			check = sum(tran(i,:),dim=1)
			do j = 1,ng
				tran(i,j) = tran(i,j) / check
			end do
		end do

		!/* stationary distribution */
		trans  = transpose(tran)
		probst = 1.0/real(ng)
		check  = 1.0
		do i = 1,10000
			if (check < 1.0e-008) exit
			probst1 = matmul(trans,probst)
			check   = maxval(abs(probst1-probst))
			probst  = probst1
		end do
		check = abs(sum(probst)-1.0)
		if (check >= 1.0e-002) then
			write(*,*) "Warning in TauchenHussey: the distribution is NOT probability measure"
		end if

		!/* normalize expected value to be one */
		z_exp = exp(node)
		z_agg = dot_product(z_exp,probst)
		value = z_exp / z_agg

	end subroutine TauchenHussey


	function norm_pdf(x,mu,sigma)
	! Purpose:
	!	probability density function.
	!
	!  Input:
	!	x          :evaluation value
	!	mu         :mean
	!	sigma      :standard deviation
	!
	!  Output:
	!	pdf        :probability density
	!
	!	External File: nrtype
	!
	!  Record of revisions:
	!	   Date      Programmer   Description of change
	!   ==========   ==========   =====================
	!	04/06/2007   T. Yamada    Original
		use nrtype,only:dp,pi
		implicit none
		!****** input ******
		real(dp),intent(in) :: x,mu,sigma
		!****** output ******
		real(dp) :: norm_pdf
		!--------------------
		norm_pdf = 1.0/sqrt(2.0*pi*sigma) * exp(-(x-mu)**2/(2.0*sigma))
	end function

end module mod_Markov
