module mod_generate_grid
!  Purpose:
!	Generate meshed grids (evaluation points).
!	The grid are uniformly spaced or exponential.
!
!  Record of revisions:
!	   Date      Programmer   Description of change
!   ==========   ==========   ======================
!   10/21/2010   T. Yamada    Current version
	implicit none
	contains

	subroutine grid_uniformly(mink,maxk,num_grid,grid)
	!  Purpose: 
	!	Generate a uniform grid between [maxk, mink].
	!	The number of grids is num_grid.
	!
	!  Record of revisions:
	!	   Date      Programmer   Description of change
	!   ==========   ==========   =====================
	!   07/19/2003   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		integer :: i
		real(dp) :: increment
		!-----------------------------
		increment = (maxk-mink)/real(num_grid-1)
		do i = 1,num_grid
			grid(i) = (i-1)*increment+mink
		end do
		if (grid(num_grid)/=maxk) then
			grid(num_grid) = maxk
		endif
	end subroutine grid_uniformly


	subroutine grid_exponential(mink,maxk,a,num_grid,grid)
	!  Purpose:
	!	Generatea a grid which is exponentially distributed.
	!	y(i) = x(i)**a
	!
	!  Record of revisions:
	!	   Date      Programmer   Description of change
	!   ==========   ==========   =====================
	!   02/27/2005   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink,a
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		integer :: i
		real(dp),dimension(num_grid) :: y,mesh
		!-----------------------------
		do i = 1,num_grid
			y(i) = real(i)**a
		end do
		do i = 1,num_grid
			mesh(i) = (y(i)-y(1))/(y(num_grid)-y(1))
		end do
		do i = 1,num_grid
			grid(i) = (maxk-mink)*mesh(i)+mink
		end do
	end subroutine grid_exponential


	subroutine grid_exp(mink,maxk,num_grid,grid)
	!  Purpose:
	!	Generatea a double exponential grid
	!	From Carroll (2006)'s Matlab code.
	!
	!  Record of revisions:
	!	   Date      Programmer   Description of change
	!   ==========   ==========   =====================
	!   10/21/2010   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		real(dp) :: maxd
		real(dp),dimension(num_grid) :: mesh
		!-----------------------------
		maxd = log(maxk+1.0)
		call grid_uniformly(mink,maxd,num_grid,mesh)
		grid = exp(mesh)-1
	end subroutine grid_exp


	subroutine grid_double_exp(mink,maxk,num_grid,grid)
	!  Purpose:
	!	Generatea a double exponential grid
	!	From Carroll (2006)'s Matlab code.
	!
	!  Record of revisions:
	!	   Date      Programmer   Description of change
	!   ==========   ==========   =====================
	!   10/21/2010   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		real(dp) :: maxd
		real(dp),dimension(num_grid) :: mesh
		!-----------------------------
		maxd = log(log(maxk+1.0)+1.0)
		call grid_uniformly(mink,maxd,num_grid,mesh)
		grid = exp(exp(mesh)-1)-1
	end subroutine grid_double_exp


	subroutine grid_triple_exp(mink,maxk,num_grid,grid)
	!  Purpose:
	!	Generatea a triple exponential grid
	!	From Carroll (2006)'s Matlab code.
	!
	!  Record of revisions:
	!	   Date      Programmer   Description of change
	!   ==========   ==========   =====================
	!   10/21/2010   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		real(dp) :: maxd
		real(dp),dimension(num_grid) :: mesh
		!-----------------------------
		maxd = log(log(log(maxk+1.0)+1.0)+1.0)
		call grid_uniformly(mink,maxd,num_grid,mesh)
		grid = exp(exp(exp(mesh)-1)-1)-1
	end subroutine grid_triple_exp

end module mod_generate_grid