module mod_preamble
!  Purpose:
!  Collects several subroutines and functions that are used in economics.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  02/02/2011   T. Yamada    Collect subroutines
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none	
	contains

	subroutine set_initial
	!  Purpose:
	!  Load variances of log-earnings/consumption profiles. 
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  11/26/2010   T. Yamada    Original
		implicit none
		!****** local variables ******
		integer :: j
		real(dp),dimension(244) :: var1,var2
		!----------------------------
		!/* hourly wage */
		open(10,file='real_wage_90_00.in')
			read(10,*) kappa
		close(10)
		!/* target profiles: means and variances
		open(21,file='var_cons_1984_1999.in')
		open(22,file='var_earning_1984_1999.in')
			read(21,*) var1
			read(22,*) var2
		close(21)
		close(22)
		do j = 1,4
			var_cons_1984_1999(25:85,j) = var1((j-1)*61+1:j*61)
			var_erng_1984_1999(25:85,j) = var2((j-1)*61+1:j*61)
		end do
	end subroutine set_initial


	subroutine wage_risk
	!  Purpose:
	!  Wage risk consists of three components:
	!  (1) fixed effect
	!  (2) persistent shock
	!  (3) transitory shock
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  08/01/2007   T. Yamada    Original code
	!  11/26/2010   T. Yamada    Modified for estimation
		use mod_Markov,only:Tauchen1986
		implicit none
		!****** local variables ******
		integer :: i
		real(dp) :: mu_tr,mu_fx
		real(dp) :: std_z,var_eps,w,mm
		real(dp),dimension(js:jr) :: std_eps
		real(dp),dimension(nz) :: pi_dist,dummy
		!-----------------------------
		if (rho < 0.0) then
			rho = 0.0
		elseif (rho > 1.0) then
			rho = 1.0
		end if
		if (std_eps20 < 0.0) then
			std_eps20 = 0.0
		end if
		if (std_pr < 0.0) then
			std_pr = 0.0
		end if
		if (sig_tr < 0.0) then
			sig_tr = 0.0
		end if
		if (sig_fx < 0.0) then
			sig_fx = 0.0
		end if
		!/* var and std of wage
		std_eps(js) = std_eps20
		var_eps = std_eps(js)**2
		std_z   = sqrt(var_eps/(1.0-(rho**2)))
		!/* Tauchen(1986)'s approximation
		mm = 1.2*log(real(nz))
		call Tauchen1986(nz,rho,mu_z,std_eps(js),std_z,mm,prob,pi_dist,dummy,endow(:,js))
		!/* persistent wage risk */
		do i = js+1,jr
			!/* linear income variance profile
			std_eps(i) = std_eps(i-1) + std_pr
			var_eps    = std_eps(i)**2
			std_z      = sqrt(var_eps/(1.0-(rho**2)))
			call Tauchen1986(nz,rho,mu_z,std_eps(i),std_z,mm,prob,pi_dist,dummy,endow(:,i))
		end do
		!/* transitory shock */
		if (nt == 1) then
			shock_tr(1) = 1.0
			prob_tr(1)  = 1.0
		elseif (nt == 2) then
			shock_tr(1) = exp(-sig_tr)
			shock_tr(2) = 1.0+(1.0-exp(-sig_tr))
			prob_tr(:)  = 0.5
		end if
		!/* permanent shock */
		if (nf == 1) then
			shock_fx(1) = 1.0
			prob_fx(1)  = 1.0
		elseif (nf == 2) then
			shock_fx(1) = exp(-sig_fx)
			shock_fx(2) = 1.0+(1.0-exp(-sig_fx))
			prob_fx(:)  = 0.5
		end if
	end subroutine wage_risk


	subroutine factor_price(KoverL,tfp,share,depre,rent,wage,KoverY)
	!  Purpose:
	!  Assume that the production function is of Cobb-Douglas type.
	!  Compute the interest rate and the wage given K over L.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  11/05/2006   Tomoaki Y.   Original code
		implicit none
		!****** input ******
		real(dp),intent(in)  :: KoverL,tfp,share,depre
		!****** output ******
		real(dp),intent(out) :: rent,wage,KoverY
		!--------------------
		rent   =      share *tfp*(KoverL**(share-1.0)) - depre
		wage   = (1.0-share)*tfp*(KoverL**share)
		KoverY = KoverL**(1.0-share)
	end subroutine factor_price


	subroutine govt_budget_ss(frac_ret,tax_ss)
	!  Purpose:
	!  The government's budget constraint.
	!  Given a replacement rate, compute an equilibrium payroll tax rate.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  11/08/2006   T. Yamada    Original
	!  02/02/2010   T. Yamada    Remove wage and H_agg
		implicit none
		!****** input ******
		real(dp),intent(in) :: frac_ret
		!****** output ******
		real(dp),intent(out) :: tax_ss
		!--------------------
		tax_ss = rep_rate*frac_ret
	end subroutine govt_budget_ss


	subroutine govt_budget_general(gov_exp,rent,wage,K_agg,H_agg,tax_lab)
	!  Purpose:
	!  The government's budget constraint.
	!  From exogenously given government expenditure
	!  and revenue from capital income tax,
	!  compute labor income tax rate.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  02/08/2011   T. Yamada    Original code
		implicit none
		!****** input ******
		real(dp),intent(in) :: gov_exp,rent,wage
		real(dp),intent(in) :: K_agg,H_agg
		!****** output ******
		real(dp),intent(out) :: tax_lab
		!****** local variables ******
		real(dp) :: revenue
		!-----------------------------
		revenue = tax_cap*rent*K_agg
		tax_lab = (gov_exp-revenue)/(wage*H_agg)
	end subroutine govt_budget_general

end module mod_preamble