subroutine middle_loop(dim,params,loglh)
!  Purpose:
!  Middle loop for estimation.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  11/26/2010   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_generate_grid
	use mod_preamble
	implicit none

	!****** input ******
	integer,intent(in)  :: dim
	real(dp),intent(in),dimension(dim) :: params

	!****** output *******
	real(dp),intent(out) :: loglh

	!****** list of variables ******
	integer :: it_gy,it_kl,i,j
	real(dp) :: metric_cap = 1.0
	real(dp) :: metric_lab = 1.0
	real(dp) :: metric_gy = 1.0
	integer,parameter :: maxit = 1000
	real(dp),parameter :: adj = 0.2
	real(dp),parameter :: tol_cap=1.0e-003
	real(dp),parameter :: tol_lab=1.0e-003
	real(dp),parameter :: tol_gy=1.0e-003

	!****** estimation ******
	real(dp) :: frisch,ave_h,var_c_all,var_y_all
	real(dp) :: ave1,ave2
	real(dp),dimension(js:jf) :: c_age,y_age
	real(dp),dimension(js:jf) :: var_c_age,var_y_age
	real(dp),dimension(25:60) :: prof1,prof2
	real(dp),dimension(25:60,2) :: lh

	!****** policy functions ******
	type(policy_function) :: policy

	!****** macro variables ******
	real(dp) :: K_agg,K_supply
	real(dp) :: H_agg,H_supply
	real(dp) :: KoverH,KoverY
	real(dp) :: Y_agg

	!****** factor prices ******
	real(dp) :: wage,rent

	!****** tax rates ******
	real(dp) :: tax_ss
	real(dp) :: tax_lab
	real(dp) :: gov_exp,gov_exp_new

	!****** accidental bequest ******
	real(dp) :: beq

	!****** age-capital profile ******
	real(dp),dimension(js:jf) :: K_age,K_mean_age,K_next_age

	!****** age-labor profile ******
	real(dp),dimension(js:jr) :: H_age,H_mean_age,H_hour_age

	!****** density function ******
	type(phi_function) :: phi
	!-------------------------------

	!/* set parameters
	rho = params(1)
	std_eps20 = params(2)
	std_pr = params(3)
	sig_tr = params(4)
	sig_fx = params(5)

	!/* display current parameters
	write(*,*) "counter:", iter
	write(*,*) "beta:  ", params(1)
	write(*,*) "std 20:", params(2)
	write(*,*) "slope: ", params(3)
	write(*,*) "std tr:", params(4)
	write(*,*) "std fx:", params(5)
	write(*,*) ""
	iter = iter+1

	!/* idiosyncratic wage risk */
	call wage_risk

	!/* initial guess of capital-labor ratio */
	K_agg = 1250.0
	H_agg =  460.0
	beq = 10.0
	KoverH = K_agg/H_agg

	!/* initial factor prices & payroll tax rate */
	call factor_price(KoverH,tfp,theta,delta,rent,wage,KoverY)
	call govt_budget_ss(frac_ret,tax_ss)

	!/* government expenditure */
	Y_agg = tfp*(K_agg**theta)*(H_agg**(1.0-theta))
	gov_exp = goy*Y_agg
	tax_lab = 0.11

	!****** middle loop: general equilibrium ******
	metric_gy = 1.0
	do it_gy = 1,maxit

		if (metric_gy<tol_gy) exit
		metric_cap = 1.0
		metric_lab = 1.0

		do it_kl = 1,maxit

			if (metric_cap<tol_cap .and. metric_lab<tol_lab) exit

			!/* household's optimizaiton
			call end_grid_method(rent,wage,tax_ss,tax_lab,H_agg,beq,policy)

			!/* stationary distribution and asset profile
			call law_of_motion(policy,phi,K_mean_age,K_next_age,H_mean_age,H_hour_age)

			!/* aggregation
			do i = js,jf
				K_age(i) = mu(i)*K_mean_age(i)
			end do
			do i = js,jr
				H_age(i) = mu(i)*H_mean_age(i)
			end do
			K_supply = sum(K_age)
			H_supply = sum(H_age)

			!/* accidental bequest
			beq = 0.0
			do i = js,jf
				beq = (1.0-surv(i))*mu(i)*K_next_age(i) + beq
			end do

			!/* iteration error
			metric_cap = abs((K_supply-K_agg)/K_supply)
			metric_lab = abs((H_supply-H_agg)/H_supply)

			!/* up-data & new factor prices
			K_agg = adj*K_supply + (1.0-adj)*K_agg
			H_agg = adj*H_supply + (1.0-adj)*H_agg
			KoverH = K_agg/H_agg
			call factor_price(KoverH,tfp,theta,delta,rent,wage,KoverY)

		end do

		Y_agg = tfp*(K_agg**theta)*(H_agg**(1.0-theta))
		gov_exp_new = goy*Y_agg
		call govt_budget_general(gov_exp_new,rent,wage,K_agg,H_agg,tax_lab)
		metric_gy = abs((gov_exp-gov_exp_new)/gov_exp)
		gov_exp = gov_exp_new

	end do

!	write(*,"(' Iteration Error: Capital,', f10.4,'%')") metric_cap*100.0
!	write(*,"(' Iteration Error: Labor,  ', f10.4,'%')") metric_lab*100.0

	!/* simulated profiles */
	call simulated_results(policy,phi,rent,wage,tax_ss,tax_lab,K_agg,H_agg,beq,var_c_age,var_y_age)

	!/* ssr: consumption
	lh = 0.0
	prof1 = var_c_age(25:60)
	prof2 = var_cons_1984_1999(25:60,1)
	ave1  = sum(var_c_age(25:60))/real(60-25+1)
	do i = 25,60
		lh(i,1) = (prof1(i)-prof2(i))**2
	end do

	!/* ssr: earning
	prof1 = var_y_age(25:60)
	prof2 = var_erng_1984_1999(25:60,1)
	ave2  = sum(var_y_age(25:60))/real(60-25+1)
	do i = 25,60
		lh(i,2) = (prof1(i)-prof2(i))**2
	end do

	!/* log-likelihood
	loglh = sum(lh(:,1),dim=1)/ave1 + sum(lh(:,2),dim=1)/ave2

end subroutine middle_loop