subroutine simulated_results(policy,phi,rent,wage,tax_ss,tax_lab,K_agg,H_agg,beq,var_c_age,var_y_age)
!  Purpose:
!  Compute average and variance paths.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  11/26/2010   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none

	!****** input ******
	type(policy_function),intent(in) :: policy
	type(phi_function),intent(in) :: phi
	real(dp),intent(in) :: rent,wage,tax_ss,tax_lab,K_agg,H_agg,beq

	!****** output ******
	real(dp),intent(out),dimension(js:jf) :: var_c_age,var_y_age

	!****** local variables ******
	integer :: i,j,k,l,ia
	real(dp) :: output,KoverY
	!------------------------------

	!/* consumption and earning profile */
	call cons_income_inequality(policy,phi,rent,wage,tax_ss,tax_lab,H_agg,beq,var_c_age,var_y_age)

	!/* macro statistics
	output = tfp*(K_agg**theta)*(H_agg**(1.0-theta))
	KoverY = K_agg/output

	!/* tentative results
	write(*,*) "Capital-Output Ratio:             ", KoverY

end subroutine simulated_results


subroutine cons_income_inequality(policy,phi,rent,wage,tax_ss,tax_lab,H_agg,beq,var_c_age,var_y_age)
!  Purpose:
!  Average and log-variance of consumption and income.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  02/06/2008   T. Yamada    Original code
!  11/08/2008   T. Yamada    Revised to submit JEDC
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_interpolation
 	implicit none

	!****** input ******
	type(policy_function),intent(in) :: policy
	type(phi_function),intent(in) :: phi
	real(dp),intent(in) :: rent,wage,tax_ss,tax_lab,H_agg,beq

	!****** output ******
	real(dp),dimension(js:jf) :: var_c_age,var_y_age

	!****** local variables ******
	integer :: i,j,k,l,ia,status
	real(dp) :: wealth
	real(dp),dimension(js:jf) :: avg_cons,avg_erng
	real(dp),dimension(js:jf) :: sqr_cons,sqr_erng
	real(dp),dimension(js:jf) :: zlc,zly
	real(dp),allocatable,dimension(:,:,:,:,:) :: log_cons1,log_erng1,hours
	real(dp),allocatable,dimension(:,:,:,:,:) :: saving1,cons1,earning1
	real(dp),allocatable,dimension(:,:,:) :: log_cons2,log_erng2
	real(dp),allocatable,dimension(:,:,:) :: saving2,cons2,earning2
	!-----------------------------

	allocate(log_cons1(nd,nz,nt,nf,js:jf),STAT=status)
	allocate(log_erng1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_cons2(nd,nf,jr+1:jf),STAT=status)
	allocate(log_erng2(nd,nf,jr+1:jf),STAT=status)
	log_cons1 = 0.0
	log_cons2 = 0.0
	log_erng1 = 0.0
	log_erng2 = 0.0

	allocate(saving1(nd,nz,nt,nf,js:jf),STAT=status)
	allocate(saving2(nd,nf,jr+1:jf),STAT=status)
	allocate(cons1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(cons2(nd,nf,jr+1:jf),STAT=status)
	allocate(earning1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(earning2(nd,nf,jr+1:jf),STAT=status)
	allocate(hours(nd,nz,nt,nf,js:jr),STAT=status)
	saving1 = 0.0
	saving2 = 0.0
	cons1 = 0.0
	cons2 = 0.0
	earning1 = 0.0
	earning2 = 0.0
	hours = 0.0

	!/* re-compute policy function over finer grids */
	call finer_policy(policy,saving1,hours,saving2)

	!/* consumption and earnings */
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							earning1(i,j,k,l,ia) = wage*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hours(i,j,k,l,ia)
							wealth               = (1.0+(1.0-tax_cap)*rent)*(grcap2(i)+beq) + (1.0-tax_ss-tax_lab)*earning1(i,j,k,l,ia)
							cons1(i,j,k,l,ia)    = wealth - tfp_gr*saving1(i,j,k,l,ia)
						end do
					end do
				end do
			end do
		case(jr+1:jf)
			do l = 1,nf
				do i = 1,nd
					earning2(i,l,ia) = wage*rep_rate*H_agg*shock_fx(l)
					wealth           = (1.0+(1.0-tax_cap)*rent)*(grcap2(i)+beq) + earning2(i,l,ia)
					cons2(i,l,ia)    = wealth - tfp_gr*saving2(i,l,ia)
				end do
			end do
		end select
	end do
	deallocate(saving1)
	deallocate(saving2)

	!/* log of consumption and earning */
	zlc = 0.0
	zly = 0.0
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							if (cons1(i,j,k,l,ia) > 0.0) then
								log_cons1(i,j,k,l,ia) = log(cons1(i,j,k,l,ia))
							else
								log_cons1(i,j,k,l,ia) = 0.0
								zlc(ia) = phi%y(i,j,k,l,ia) + zlc(ia)
							end if
							if (earning1(i,j,k,l,ia) > 0.0) then
								log_erng1(i,j,k,l,ia) = log(earning1(i,j,k,l,ia))
							else
								log_erng1(i,j,k,l,ia) = 0.0
								zly(ia) = phi%y(i,j,k,l,ia) + zly(ia)
							end if
						end do
					end do
				end do
			end do
		case(jr+1:jf)
			do l = 1,nf
				do i = 1,nd
					if (cons2(i,l,ia) > 0) then
						log_cons2(i,l,ia) = log(cons2(i,l,ia))
					else
						log_cons2(i,l,ia) = 0.0
						zlc(ia) = phi%y(i,j,k,l,ia) + zlc(ia)
					end if
					if (earning2(i,l,ia) > 0) then
						log_erng2(i,l,ia) = log(earning2(i,l,ia))
					else
						log_erng2(i,l,ia) = 0.0
						zly(ia) = phi%y(i,j,k,l,ia) + zly(ia)
					end if
				end do
			end do
		end select
	end do

	!/* variances of log-consumption etc. by age */
	sqr_cons = 0.0
	sqr_erng = 0.0
	avg_cons = 0.0
	avg_erng = 0.0
	do l = 1,nf
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							sqr_cons(ia) = phi%y(i,j,k,l,ia)*(log_cons1(i,j,k,l,ia)**2) + sqr_cons(ia)
							sqr_erng(ia) = phi%y(i,j,k,l,ia)*(log_erng1(i,j,k,l,ia)**2) + sqr_erng(ia)
							avg_cons(ia) = phi%y(i,j,k,l,ia)*log_cons1(i,j,k,l,ia) + avg_cons(ia)
							avg_erng(ia) = phi%y(i,j,k,l,ia)*log_erng1(i,j,k,l,ia) + avg_erng(ia)
						end do
					end do
				end do
				sqr_cons(ia) = sqr_cons(ia) / (1.0-zlc(ia))
				sqr_erng(ia) = sqr_erng(ia) / (1.0-zly(ia))
				avg_cons(ia) = avg_cons(ia) / (1.0-zlc(ia))
				avg_erng(ia) = avg_erng(ia) / (1.0-zly(ia))
			case(jr+1:jf)
				do i = 1,nd
					sqr_cons(ia) = phi%o(i,l,ia)*(log_cons2(i,l,ia)**2) + sqr_cons(ia)
					sqr_erng(ia) = phi%o(i,l,ia)*(log_erng2(i,l,ia)**2) + sqr_erng(ia)
					avg_cons(ia) = phi%o(i,l,ia)*log_cons2(i,l,ia) + avg_cons(ia)
					avg_erng(ia) = phi%o(i,l,ia)*log_erng2(i,l,ia) + avg_erng(ia)
				end do
				sqr_cons(ia) = sqr_cons(ia) / (1.0-zlc(ia))
				sqr_erng(ia) = sqr_erng(ia) / (1.0-zly(ia))
				avg_cons(ia) = avg_cons(ia) / (1.0-zlc(ia))
				avg_erng(ia) = avg_erng(ia) / (1.0-zly(ia))
			end select
		end do
	end do
	var_c_age = 0.0
	var_y_age = 0.0
	do ia = js,jf
		var_c_age(ia) = sqr_cons(ia) - avg_cons(ia)**2
		var_y_age(ia) = sqr_erng(ia) - avg_erng(ia)**2
	end do

	deallocate(log_cons1)
	deallocate(log_cons2)
	deallocate(log_erng1)
	deallocate(log_erng2)
	deallocate(cons1)
	deallocate(cons2)
	deallocate(earning1)
	deallocate(earning2)
	deallocate(hours)

end subroutine cons_income_inequality