module mod_EulerEq
!  Purpose:
!  RHS of the Euler equation.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  01/13/2007   T. Yamada    Original code
!  02/24/2007   T. Yamada    Modified to include TFP
!  11/08/2008   T. Yamada    Revised to submit JEDC
!  02/02/2011   T. Yamada    Revised to resubmit JEDC
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none
	contains

	function mv_worker(asset,age,time,zt,fx,rent,wage,tax_ss,tax_lab,beq,tfp_gr,hmax,tax_cap,cons_fcn,xgrid)
	!  Purpose:
	!  Right hand side of Euler equation for working households.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/08/2007   T. Yamada    Original code
	!  11/08/2008   T. Yamada    Revised to submit JEDC
		use mod_interpolation,only:interp1
		implicit none
		!****** input ******
		real(dp),intent(in) :: asset,rent,wage,tax_ss,tax_lab,beq,tfp_gr,hmax,tax_cap
		integer,intent(in) :: age,time,zt,fx
		real(dp),intent(in),dimension(0:na,nz,nt,nf,js:jr) :: cons_fcn
		real(dp),intent(in),dimension(0:na,nz,nt,nf,js:jr) :: xgrid
		!****** output ******
		real(dp) :: mv_worker
		!****** local variables ******
		integer :: j,k
		real(dp) :: exp_value,beta_tilde,earning
		real(dp),dimension(nz) :: exp_value_z
		real(dp),dimension(nz,nt) :: coh,cons,labor,mu
		!-----------------------------
		coh   = 0.0
		cons  = 0.0
		labor = 0.0
		mu    = 0.0
		do k = 1,nt
			do j = 1,nz
				earning  = (1.0-tax_ss-tax_lab)*wage*kappa(age+1)*endow(j,age+1)*shock_tr(k)*shock_fx(fx)*hmax
				coh(j,k) = (1.0+(1.0-tax_cap)*rent)*(asset+beq) + earning
				!/* approximate consumption
				cons(j,k)  = consumption_fcn(coh(j,k),age+1,cons_fcn(:,j,k,fx,age+1),xgrid(:,j,k,fx,age+1))
				if (cons(j,k) > coh(j,k)) then
					cons(j,k) = coh(j,k)
				end if
				!/* labor suppy
				labor(j,k) = labor_from_cons(cons(j,k),wage,kappa(age+1),endow(j,age+1),shock_tr(k),shock_fx(fx),tax_ss,tax_lab,hmax)
				!/* marginal utility
				mu(j,k) = marg_util(cons(j,k),labor(j,k),hmax)
			end do
		end do
		!/* expected value
		do j = 1,nz
			exp_value_z(j) = dot_product(prob_tr,mu(j,:))
		end do
		exp_value  = dot_product(prob(zt,:),exp_value_z)
		!/* marginal value
		beta_tilde = beta*(tfp_gr**(sigma*(1.0-gamma)-1.0))
		mv_worker  = surv(age,time)*beta_tilde*(1.0+(1.0-tax_cap)*rent)*exp_value
	end function mv_worker


	function mv_retire(asset,age,time,fx,rent,wage,H_agg,beq,tfp_gr,hmax,tax_cap,cons_fcn,xgrid)
	!  Purpose:
	!  Right hand side of the Euler equation for retired households.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/08/2007   T. Yamada    Original code
	!  11/08/2008   T. Yamada    Revised to submit JEDC
		implicit none
		!****** input ******
		real(dp),intent(in) :: asset,rent,wage,H_agg,beq,tfp_gr,hmax,tax_cap
		integer,intent(in) :: age,time,fx
		real(dp),intent(in),dimension(0:na,nf,jr+1:jf) :: cons_fcn
		real(dp),intent(in),dimension(0:na,nf,jr+1:jf) :: xgrid
		!****** output ******
		real(dp) :: mv_retire
		!****** local variables ******
		integer :: i,j
		real(dp) :: ss,coh,cons,labor
		real(dp) :: exp_value,beta_tilde
		!-----------------------------
		!/* cash on hand
		ss = rep_rate*wage*H_agg*shock_fx(fx)
		coh = (1.0+(1.0-tax_cap)*rent)*(asset+beq) + ss
		!/* policy functions
		labor = 0.0
		cons  = consumption_fcn(coh,age+1,cons_fcn(:,fx,age+1),xgrid(:,fx,age+1))
		!/* marginal utility
		exp_value = marg_util(cons,labor,hmax)
		!/* marginal value
		beta_tilde = beta*(tfp_gr**(sigma*(1.0-gamma)-1.0))
		mv_retire  = surv(age,time)*beta_tilde*(1.0+(1.0-tax_cap)*rent)*exp_value
	end function mv_retire


	function marg_util(cons,labor,hmax)
	!  Purpose:
	!  Marginal utility of consumption,
	!  when utility function is Cobb-Douglas type.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/08/2007   T. Yamada    Original code
	!  11/10/2008   T. Yamada    log-log case
		implicit none
		!****** input ******
		real(dp),intent(in) :: cons,labor,hmax
		!****** output ******
		real(dp) :: marg_util
		!****** local variables ******
		real(dp) :: utilc,utill,indic
		!-----------------------------
		if (gamma == 1) then
			marg_util = 1.0/cons
		else
			if (labor > 0.0) then
				indic = 1.0
			else
				indic = 0.0
			end if
			utilc = cons**sigma
			utill = (hmax-labor-psi*indic)**(1.0-sigma)
			marg_util = sigma*((utilc*utill)**(1.0-gamma))/cons
		end if
	end function marg_util


	function consumption_fcn(coh,age,policy,xgrid)
	!  Purpose:
	!  Consumption function over the cash on hand.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/08/2007   Tomoaki Y.   Original code
		use mod_interpolation,only:interp1
		implicit none
		!****** input ******
		real(dp),intent(in) :: coh
		integer,intent(in) :: age
		real(dp),intent(in),dimension(0:na) :: policy,xgrid
		!****** output ******
		real(dp) :: consumption_fcn
		!--------------------
		if (coh < xgrid(1)) then
			if (age > jr) then
				consumption_fcn = coh
			elseif (age <= jr) then
				consumption_fcn = sigma*(coh-amin)
!				consumption_fcn = interp1(xgrid(1:na),policy(1:na),coh)
			end if
		elseif (coh > xgrid(na)) then
			consumption_fcn = interp1(xgrid(1:na),policy(1:na),coh)
		else
			consumption_fcn = interp1(xgrid(1:na),policy(1:na),coh)
		end if
	end function consumption_fcn


	function inverse_mu_labor(rhs,age,zt,tr,fx,wage,tax_ss,tax_lab)
	!  Purpose:
	!  Inverse of the marginal utility of consumption.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/14/2007   T. Yamada    Original code
	!  11/10/2008   T. Yamada    Include log utility
		implicit none
		!****** input ******
		real(dp),intent(in) :: rhs,wage,tax_ss,tax_lab
		integer,intent(in) :: zt,age,tr,fx
		!****** output ******
		real(dp) :: inverse_mu_labor
		!****** local variables ******
		real(dp) :: index,ww,const
		!-----------------------------
		if (gamma == 1.0) then
			inverse_mu_labor = 1.0/rhs
		else
			index = (1.0-sigma)*(1.0-gamma)
			ww    = (1.0-tax_ss-tax_lab)*wage*kappa(age)*endow(zt,age)*shock_tr(tr)*shock_fx(fx)
			const = rhs/(sigma*(((1.0-sigma)/(sigma*ww))**index))
			index = -1.0/gamma
			inverse_mu_labor = const**index
		end if
	end function inverse_mu_labor


	function inverse_mu_zero_labor(rhs,hmax)
	!  Purpose:
	!  Inverse of the marginal utility of consumption,
	!  when labor supply is zero.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/08/2007   T. Yamada    Original code
	!  11/10/2008   T. Yamada    Include log utility
		implicit none
		!****** input ******
		real(dp),intent(in) :: rhs,hmax
		!****** output ******
		real(dp) :: inverse_mu_zero_labor
		!****** local variables ******
		real(dp) :: labor,index,const
		!-----------------------------
		if (gamma == 1.0) then
			inverse_mu_zero_labor = 1.0/rhs
		else
			labor = 0.0
			index = (1.0-sigma)*(1.0-gamma)
			const = rhs/(sigma*((hmax-labor)**index))
			index = 1.0/(sigma*(1.0-gamma)-1.0)
			inverse_mu_zero_labor = const**index
		end if
	end function inverse_mu_zero_labor


	function labor_from_cons(cons,wage,kappa,endow,shock_tr,shock_fx,tax_ss,tax_lab,hmax)
	!  Purpose:
	!  Compute labor supply from consumption.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/09/2007   T. Yamada    Original code
		implicit none
		!****** input ******
		real(dp),intent(in) :: cons,wage,kappa,endow,shock_tr,shock_fx,tax_ss,tax_lab,hmax
		!****** output ******
		real(dp) :: labor_from_cons
		!****** local variables ******
		real(dp) :: ww
		!-----------------------------
		ww = (1.0-tax_ss-tax_lab)*wage*kappa*endow*shock_tr*shock_fx
		labor_from_cons = hmax - ((1.0-sigma)/sigma)*(cons/ww) - psi
		if (labor_from_cons > hmax) then
		    labor_from_cons = hmax
		elseif (labor_from_cons < 0.0) then
		    labor_from_cons = 0.0
		end if
	end function labor_from_cons

end module mod_EulerEq