module mod_end_grid_method
!  Purpose:
!  Compute policy functions usint the endogenous gridpoint method.
!  This module contains all subroutines and functions
!  to compute policy functions of steady states and transition.
!
!  Record of Revisions:
!     Date      Programmer   Description of Change
!  ==========   ==========   =====================
!  02/03/2011   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none
	contains

	subroutine end_grid_method_ss(time,rent,wage,tax_ss,tax_lab,H_agg,beq,tfp_gr,hmax,theta,delta,tax_cap,policy,egm1)
	!  Purpose:
	!  Compute policy functions using the Endogenous Grid Method.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/08/2007   T. Yamada    Original code
	!  11/08/2008   T. Yamada    Revised to submit JEDC
	!  02/03/2011   T. Yamada    Revised to resubmit JEDC
		use mod_interpolation,only:interp1
		use mod_EulerEq
		implicit none

		!****** input ******
		integer,intent(in) :: time
		real(dp),intent(in) :: rent,wage,tax_ss,tax_lab,H_agg,beq
		real(dp),intent(in) :: tfp_gr,hmax,theta,delta,tax_cap

		!****** output ******
		type(policy_function),intent(out) :: policy
		type(policy_egm),intent(out) :: egm1

		!****** local variables ******
		integer :: i,j,k,l,ia,status
		real(dp) :: rhs,c_tilde,cons,earning,ss
		real(dp),dimension(na) :: a_prime
		real(dp),allocatable,dimension(:,:,:,:,:) :: asset1
		real(dp),allocatable,dimension(:,:,:) :: asset2
		!-----------------------------

		!/* current financial asset
		allocate(asset1(na,nz,nt,nf,js:jr),STAT=status)
		allocate(asset2(na,nf,jr+1:jf),STAT=status)
		asset1 = 0.0
		asset2 = 0.0

		!/* next period's choice
		a_prime = grcap1

		!/* consumption function over cash on hand */
		do l = 1,nf
			do ia = jf,js,-1
				select case (ia)
				case(jf)
					egm1%co(0,l,ia) = 0.0
					egm1%go(0,l,ia) = 0.0
					ss = rep_rate*wage*H_agg*shock_fx(l)
					egm1%go(1:na,l,ia) = (1.0+(1.0-tax_cap)*rent)*(grcap1+beq) + ss
					egm1%co(1:na,l,ia) = egm1%go(1:na,l,ia)
					asset2(1:na,l,ia)  = grcap1
				case(jr+1:jf-1)
					egm1%co(0,:,ia) = 0.0
					egm1%go(0,:,ia) = 0.0
					do i = 1,na
						!/* marginal value
						rhs = mv_retire(a_prime(i),ia,time,l,rent,wage,H_agg,beq,tfp_gr,hmax,tax_cap,egm1%co,egm1%go)
						!/* inverse of marginal value
						egm1%co(i,l,ia) = inverse_mu_zero_labor(rhs,hmax)
						egm1%go(i,l,ia) = egm1%co(i,l,ia) + tfp_gr*a_prime(i)
					end do
					ss = rep_rate*wage*H_agg*shock_fx(l)
					!/* retrieve current financial asset
					asset2(:,l,ia) = (egm1%go(1:na,l,ia)-ss)/(1.0+(1.0-tax_cap)*rent)-beq
				case(js:jr)
					egm1%cy(0,:,:,:,ia) = 0.0
					egm1%gy(0,:,:,:,ia) = 0.0
					do k = 1,nt
						do j = 1,nz
							do i = 1,na
								!/* marginal value
								if (ia == jr) then
									rhs = mv_retire(a_prime(i),ia,time,l,rent,wage,H_agg,beq,tfp_gr,hmax,tax_cap,egm1%co,egm1%go)
								else
									rhs = mv_worker(a_prime(i),ia,time,j,l,rent,wage,tax_ss,tax_lab,beq,tfp_gr,hmax,tax_cap,egm1%cy,egm1%gy)
								end if
								!/* inverse of marginal value
								egm1%cy(i,j,k,l,ia) = inverse_mu_labor(rhs,ia,j,k,l,wage,tax_ss,tax_lab)
								egm1%wh(i,j,k,l,ia) = labor_from_cons(egm1%cy(i,j,k,l,ia),wage,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss,tax_lab,hmax)
								if (egm1%wh(i,j,k,l,ia) == 0.0) then
									egm1%cy(i,j,k,l,ia) = inverse_mu_zero_labor(rhs,hmax)
								end if
								c_tilde = egm1%cy(i,j,k,l,ia) + (1.0-tax_ss-tax_lab)*wage*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*(hmax-egm1%wh(i,j,k,l,ia))
								egm1%gy(i,j,k,l,ia) = c_tilde + tfp_gr*a_prime(i)
							end do
							earning = (1.0-tax_ss-tax_lab)*wage*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hmax
							!/* retrieve current financial asset
							asset1(:,j,k,l,ia) = (egm1%gy(1:na,j,k,l,ia)-earning)/(1.0+(1.0-tax_cap)*rent)-beq
						end do
					end do
				end select
			end do
		end do

		!/* policy function over asset */
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do l = 1,nf
					do k = 1,nt
						do j = 1,nz
							!/* next asset
							do i = 1,na
								if (asset1(1,j,k,l,ia) > grcap1(i)) then
									policy%y(i,j,k,l,ia) = amin
								else
									policy%y(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),a_prime,grcap1(i))
								end if
								if (policy%y(i,j,k,l,ia) < amin) then
									policy%y(i,j,k,l,ia) = amin
								end if
							end do
							!/* hours worked
							do i = 1,na
								if (asset1(1,j,k,l,ia) > grcap1(i)) then
									cons = interp1(asset1(:,j,k,l,ia),egm1%cy(1:na,j,k,l,ia),grcap1(i))
									policy%l(i,j,k,l,ia) = labor_from_cons(cons,wage,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss,tax_lab,hmax)
								else
									policy%l(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),egm1%wh(:,j,k,l,ia),grcap1(i))
								end if
								if (policy%l(i,j,k,l,ia) > hmax) then
									policy%l(i,j,k,l,ia) = hmax
								elseif (policy%l(i,j,k,l,ia) < 0.0) then
									policy%l(i,j,k,l,ia) = 0.0
								end if
							end do
						end do
					end do
				end do
			case(jr+1:jf-1)
				do l = 1,nf
					do i = 1,na
						if (asset2(1,l,ia) > grcap1(i)) then
							policy%o(i,l,ia) = amin
						else
							policy%o(i,l,ia) = interp1(asset2(:,l,ia),a_prime,grcap1(i))
						end if
						if (policy%o(i,l,ia) < amin) then
							policy%o(i,l,ia) = amin
						end if
					end do
				end do
			case(jf)
				policy%o(:,:,ia) = 0.0
			end select
		end do

		deallocate(asset1)
		deallocate(asset2)

	end subroutine end_grid_method_ss


	subroutine end_grid_method_tr(time,egm0,egm1,policy)
	!  Purpose:
	!  Compute policy function for each time using endogenous gridpoint method.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  01/13/2007   T. Yamada    Original
	!  02/24/2007   T. Yamada    Modified to include TFP
	!  11/08/2008   T. Yamada    Revised to submit JEDC
	!  02/03/2011   T. Yamada    Revised to resubmit JEDC
		use mod_interpolation,only:interp1
		use mod_EulerEq
		implicit none

		!****** input ******
		integer,intent(in) :: time
		type(policy_egm),intent(in) :: egm0

		!****** output ******
		type(policy_egm),intent(out) :: egm1
		type(policy_function),intent(out) :: policy

		!****** local variables ******
		real(dp),dimension(na) :: a_prime
		real(dp),dimension(na,nz,nt,nf,js:jr) :: asset1
		real(dp),dimension(na,nf,jr+1:jf) :: asset2
		integer :: i,j,k,l,ia
		real(dp) :: rhs,c_tilde,cons,earning,ss
		real(dp) :: rent0,wage0,tax_ss0,tax_lab0,beq0,H_agg0,tfp_gr0,hmax0,tax_cap0
		real(dp) :: rent1,wage1,tax_ss1,tax_lab1,beq1,H_agg1,hmax1,tax_cap1
		!-----------------------------

		a_prime = grcap1

		!/* current factor prices etc.
		tfp_gr0  = state0(time)%tfp_gr
		hmax0    = state0(time)%hmax
		tax_cap0 = state0(time)%tax_cap
		rent0    = state0(time)%rent
		wage0    = state0(time)%wage
		tax_ss0  = state0(time)%tax_ss
		tax_lab0 = state0(time)%tax_lab
		H_agg0   = state0(time)%lab
		beq0     = state0(time)%beq

		!/* perfect foresight paths
		if (time /= tf) then
			hmax1    = state0(time+1)%hmax
			tax_cap1 = state0(time+1)%tax_cap
			rent1    = state0(time+1)%rent
			wage1    = state0(time+1)%wage
			tax_ss1  = state0(time+1)%tax_ss
			tax_lab1 = state0(time+1)%tax_lab
			H_agg1   = state0(time+1)%lab
			beq1     = state0(time+1)%beq
		else
			hmax1    = state0(time)%hmax
			tax_cap1 = state0(time)%tax_cap
			rent1    = state0(time)%rent
			wage1    = state0(time)%wage
			tax_ss1  = state0(time)%tax_ss
			tax_lab1 = state0(time)%tax_lab
			H_agg1   = state0(time)%lab
			beq1     = state0(time)%beq
		end if

		!*** consumption function over cash on hand ***
		do ia = jf,js,-1
			select case (ia)
			case(jf)
				do l = 1,nf
					egm1%co(0,l,ia) = 0.0
					egm1%go(0,l,ia) = 0.0
					ss = rep_rate*wage0*H_agg0*shock_fx(l)
					egm1%go(1:na,l,ia) = (1.0+(1.0-tax_cap0)*rent0)*(grcap1+beq0) + ss
					egm1%co(1:na,l,ia) = egm1%go(1:na,l,ia)
					asset2(1:na,l,ia)  = grcap1
				end do
			case(jr+1:jf-1)
				egm1%co(0,:,ia) = 0.0
				egm1%go(0,:,ia) = 0.0
				do l = 1,nf
					do i = 1,na
						!/* marginal value of expected utility
						rhs = mv_retire(a_prime(i),ia,time,l,rent1,wage1,H_agg1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%co,egm0%go)
						egm1%co(i,l,ia) = inverse_mu_zero_labor(rhs,hmax0)
						egm1%go(i,l,ia) = egm1%co(i,l,ia) + tfp_gr0*a_prime(i)
					end do
					ss = rep_rate*wage0*H_agg0*shock_fx(l)
					!/* retrieve current financial asset
					asset2(:,l,ia) = (egm1%go(1:na,l,ia)-ss)/(1.0+(1.0-tax_cap0)*rent0) - beq0
				end do
			case(js:jr)
				egm1%cy(0,:,:,:,ia) = 0.0
				egm1%gy(0,:,:,:,ia) = 0.0
				do l = 1,nf
					do k = 1,nt
						do j = 1,nz
							do i = 1,na
								!/* marginal value
								if (ia == jr) then
									rhs = mv_retire(a_prime(i),ia,time,l,rent1,wage1,H_agg1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%co,egm0%go)
								else
									rhs = mv_worker(a_prime(i),ia,time,j,l,rent1,wage1,tax_ss1,tax_lab1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%cy,egm0%gy)
								end if
								egm1%cy(i,j,k,l,ia) = inverse_mu_labor(rhs,ia,j,k,l,wage0,tax_ss0,tax_lab0)
								egm1%wh(i,j,k,l,ia) = labor_from_cons(egm1%cy(i,j,k,l,ia),wage0,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss0,tax_lab0,hmax0)
								if (egm1%wh(i,j,k,l,ia) == 0.0) then
									egm1%cy(i,j,k,l,ia) = inverse_mu_zero_labor(rhs,hmax0)
								end if
								c_tilde = egm1%cy(i,j,k,l,ia) + (1.0-tax_ss0-tax_lab0)*wage0*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*(hmax0-egm1%wh(i,j,k,l,ia))
								egm1%gy(i,j,k,l,ia) = c_tilde + tfp_gr0*a_prime(i)
							end do
							earning = (1.0-tax_ss0-tax_lab0)*wage0*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hmax0
							!/* retrieve current financial asset
							asset1(:,j,k,l,ia) = (egm1%gy(1:na,j,k,l,ia)-earning)/(1.0+(1.0-tax_cap0)*rent0) - beq0
						end do
					end do
				end do
			end select
		end do

		!/* policy function over asset */
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do l = 1,nf
					do k = 1,nt
						do j = 1,nz
							do i = 1,na
								if (asset1(1,j,k,l,ia) > grcap1(i)) then
									policy%y(i,j,k,l,ia) = amin
								else
									policy%y(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),a_prime,grcap1(i))
								end if
								if (policy%y(i,j,k,l,ia) < amin) then
									policy%y(i,j,k,l,ia) = amin
								end if
							end do
							do i = 1,na
								if (asset1(1,j,k,l,ia) > grcap1(i)) then
									cons = interp1(asset1(:,j,k,l,ia),egm1%cy(1:na,j,k,l,ia),grcap1(i))
									policy%l(i,j,k,l,ia) = labor_from_cons(cons,wage0,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss0,tax_lab0,hmax0)
								else
									policy%l(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),egm1%wh(:,j,k,l,ia),grcap1(i))
								end if
								if (policy%l(i,j,k,l,ia) > hmax0) then
									policy%l(i,j,k,l,ia) = hmax0
								elseif (policy%l(i,j,k,l,ia) < 0.0) then
									policy%l(i,j,k,l,ia) = 0.0
								end if
							end do
						end do
					end do
				end do
			case(jr+1:jf-1)
				do l = 1,nf
					do i = 1,na
						if (asset2(1,l,ia) > grcap1(i)) then
							policy%o(i,l,ia) = amin
						else
							policy%o(i,l,ia) = interp1(asset2(:,l,ia),a_prime,grcap1(i))
						end if
						if (policy%o(i,l,ia) < amin) then
							policy%o(i,l,ia) = amin
						end if
					end do
				end do
			case(jf)
				policy%o(:,:,ia) = 0.0
			end select
		end do

	end subroutine end_grid_method_tr


	subroutine end_grid_method_pre(time,egm0,egm1,policy)
	!  Purpose:
	!  Compute policy function for each time using endogenous gridpoint method.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  01/13/2007   T. Yamada    Original
	!  02/24/2007   T. Yamada    Modified to include TFP
	!  11/08/2008   T. Yamada    Revised to submit JEDC
	!  02/03/2011   T. Yamada    Revised to resubmit JEDC
		use mod_interpolation,only:interp1
		use mod_EulerEq
		implicit none

		!****** input ******
		integer,intent(in) :: time
		type(policy_egm),intent(in) :: egm0

		!****** output ******
		type(policy_egm),intent(out) :: egm1
		type(policy_function),intent(out) :: policy

		!****** local variables ******
		real(dp),dimension(na) :: a_prime
		real(dp),dimension(na,nz,nt,nf,js:jr) :: asset1
		real(dp),dimension(na,nf,jr+1:jf) :: asset2
		integer :: i,j,k,l,ia
		real(dp) :: rhs,c_tilde,cons,earning,ss
		real(dp) :: rent0,wage0,tax_ss0,tax_lab0,beq0,H_agg0,tfp_gr0,hmax0,tax_cap0
		real(dp) :: rent1,wage1,tax_ss1,tax_lab1,beq1,H_agg1,hmax1,tax_cap1
		!-----------------------------

		a_prime = grcap1

		!/* current factor prices etc.
		tfp_gr0  = state0(time)%tfp_gr
		hmax0    = state0(time)%hmax
		tax_cap0 = state0(time)%tax_cap
		rent0    = state0(time)%rent
		wage0    = state0(time)%wage
		tax_ss0  = state0(time)%tax_ss
		tax_lab0 = state0(time)%tax_lab
		H_agg0   = state0(time)%lab
		beq0     = state0(time)%beq

		!/* mis-predicted paths
		hmax1    = predict(time+1)%hmax
		rent1    = predict(time+1)%rent
		wage1    = predict(time+1)%wage
		tax_lab1 = predict(time+1)%tax_lab
		tax_cap1 = predict(time+1)%tax_cap
		tax_ss1  = predict(time+1)%tax_ss
		H_agg1   = predict(time+1)%lab
		beq1     = predict(time+1)%beq

		!*** consumption function over cash on hand ***
		do ia = jf,js,-1
			select case (ia)
			case(jf)
				do l = 1,nf
					egm1%co(0,l,ia) = 0.0
					egm1%go(0,l,ia) = 0.0
					ss = rep_rate*wage0*H_agg0*shock_fx(l)
					egm1%go(1:na,l,ia) = (1.0+(1.0-tax_cap0)*rent0)*(grcap1+beq0) + ss
					egm1%co(1:na,l,ia) = egm1%go(1:na,l,ia)
					asset2(1:na,l,ia)  = grcap1
				end do
			case(jr+1:jf-1)
				egm1%co(0,:,ia) = 0.0
				egm1%go(0,:,ia) = 0.0
				do l = 1,nf
					do i = 1,na
						!/* marginal value of expected utility
						rhs = mv_retire(a_prime(i),ia,time,l,rent1,wage1,H_agg1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%co,egm0%go)
						egm1%co(i,l,ia) = inverse_mu_zero_labor(rhs,hmax0)
						egm1%go(i,l,ia) = egm1%co(i,l,ia) + tfp_gr0*a_prime(i)
					end do
					ss = rep_rate*wage0*H_agg0*shock_fx(l)
					!/* retrieve current financial asset
					asset2(:,l,ia) = (egm1%go(1:na,l,ia)-ss)/(1.0+(1.0-tax_cap0)*rent0) - beq0
				end do
			case(js:jr)
				egm1%cy(0,:,:,:,ia) = 0.0
				egm1%gy(0,:,:,:,ia) = 0.0
				do l = 1,nf
					do k = 1,nt
						do j = 1,nz
							do i = 1,na
								!/* marginal value
								if (ia == jr) then
									rhs = mv_retire(a_prime(i),ia,time,l,rent1,wage1,H_agg1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%co,egm0%go)
								else
									rhs = mv_worker(a_prime(i),ia,time,j,l,rent1,wage1,tax_ss1,tax_lab1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%cy,egm0%gy)
								end if
								egm1%cy(i,j,k,l,ia) = inverse_mu_labor(rhs,ia,j,k,l,wage0,tax_ss0,tax_lab0)
								egm1%wh(i,j,k,l,ia) = labor_from_cons(egm1%cy(i,j,k,l,ia),wage0,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss0,tax_lab0,hmax0)
								if (egm1%wh(i,j,k,l,ia) == 0.0) then
									egm1%cy(i,j,k,l,ia) = inverse_mu_zero_labor(rhs,hmax0)
								end if
								c_tilde = egm1%cy(i,j,k,l,ia) + (1.0-tax_ss0-tax_lab0)*wage0*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*(hmax0-egm1%wh(i,j,k,l,ia))
								egm1%gy(i,j,k,l,ia) = c_tilde + tfp_gr0*a_prime(i)
							end do
							earning = (1.0-tax_ss0-tax_lab0)*wage0*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hmax0
							!/* retrieve current financial asset
							asset1(:,j,k,l,ia) = (egm1%gy(1:na,j,k,l,ia)-earning)/(1.0+(1.0-tax_cap0)*rent0) - beq0
						end do
					end do
				end do
			end select
		end do

		!/* policy function over asset */
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do l = 1,nf
					do k = 1,nt
						do j = 1,nz
							do i = 1,na
								if (asset1(1,j,k,l,ia) > grcap1(i)) then
									policy%y(i,j,k,l,ia) = amin
								else
									policy%y(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),a_prime,grcap1(i))
								end if
								if (policy%y(i,j,k,l,ia) < amin) then
									policy%y(i,j,k,l,ia) = amin
								end if
							end do
							do i = 1,na
								if (asset1(1,j,k,l,ia) > grcap1(i)) then
									cons = interp1(asset1(:,j,k,l,ia),egm1%cy(1:na,j,k,l,ia),grcap1(i))
									policy%l(i,j,k,l,ia) = labor_from_cons(cons,wage0,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss0,tax_lab0,hmax0)
								else
									policy%l(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),egm1%wh(:,j,k,l,ia),grcap1(i))
								end if
								if (policy%l(i,j,k,l,ia) > hmax0) then
									policy%l(i,j,k,l,ia) = hmax0
								elseif (policy%l(i,j,k,l,ia) < 0.0) then
									policy%l(i,j,k,l,ia) = 0.0
								end if
							end do
						end do
					end do
				end do
			case(jr+1:jf-1)
				do l = 1,nf
					do i = 1,na
						if (asset2(1,l,ia) > grcap1(i)) then
							policy%o(i,l,ia) = amin
						else
							policy%o(i,l,ia) = interp1(asset2(:,l,ia),a_prime,grcap1(i))
						end if
						if (policy%o(i,l,ia) < amin) then
							policy%o(i,l,ia) = amin
						end if
					end do
				end do
			case(jf)
				policy%o(:,:,ia) = 0.0
			end select
		end do

	end subroutine end_grid_method_pre


	subroutine end_grid_method_miss(time,egm0,egm1,policy)
	!  Purpose:
	!  Compute policy function for each time using endogenous gridpoint method.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  01/13/2007   T. Yamada    Original
	!  02/24/2007   T. Yamada    Modified to include TFP
	!  11/08/2008   T. Yamada    Revised to submit JEDC
	!  02/28/2011   T. Yamada    Revised to resubmit JEDC
		use mod_interpolation,only:interp1
		use mod_EulerEq
		implicit none

		!****** input ******
		integer,intent(in) :: time
		type(policy_egm),intent(in) :: egm0

		!****** output ******
		type(policy_egm),intent(out) :: egm1
		type(policy_function),intent(out) :: policy

		!****** local variables ******
		real(dp),dimension(na) :: a_prime
		real(dp),dimension(na,nz,nt,nf,js:jr) :: asset1
		real(dp),dimension(na,nf,jr+1:jf) :: asset2
		integer :: i,j,k,l,ia
		real(dp) :: rhs,c_tilde,cons,earning,ss
		real(dp) :: rent0,wage0,tax_ss0,tax_lab0,beq0,H_agg0,tfp_gr0,hmax0,tax_cap0
		real(dp) :: rent1,wage1,tax_ss1,tax_lab1,beq1,H_agg1,hmax1,tax_cap1
		!-----------------------------

		a_prime = grcap1

		!/* current factor prices etc.
		hmax0    = state0(time)%hmax
		tax_cap0 = state0(time)%tax_cap
		rent0    = state0(time)%rent
		wage0    = state0(time)%wage
		tax_ss0  = state0(time)%tax_ss
		tax_lab0 = state0(time)%tax_lab
		H_agg0   = state0(time)%lab
		beq0     = state0(time)%beq
		
		!/* mis-predicted TFP
		tfp_gr0  = predict(time)%tfp_gr

		!/* mis-predicted future paths
		tfp_gr0  = predict(time)%tfp_gr
		hmax1    = predict(time+1)%hmax
		rent1    = predict(time+1)%rent
		wage1    = predict(time+1)%wage
		tax_lab1 = predict(time+1)%tax_lab
		tax_cap1 = predict(time+1)%tax_cap
		tax_ss1  = predict(time+1)%tax_ss
		H_agg1   = predict(time+1)%lab
		beq1     = predict(time+1)%beq

		!*** consumption function over cash on hand ***
		do ia = jf,js,-1
			select case (ia)
			case(jf)
				do l = 1,nf
					egm1%co(0,l,ia) = 0.0
					egm1%go(0,l,ia) = 0.0
					ss = rep_rate*wage0*H_agg0*shock_fx(l)
					egm1%go(1:na,l,ia) = (1.0+(1.0-tax_cap0)*rent0)*(grcap1+beq0) + ss
					egm1%co(1:na,l,ia) = egm1%go(1:na,l,ia)
					asset2(1:na,l,ia)  = grcap1
				end do
			case(jr+1:jf-1)
				egm1%co(0,:,ia) = 0.0
				egm1%go(0,:,ia) = 0.0
				do l = 1,nf
					do i = 1,na
						!/* marginal value of expected utility
						rhs = mv_retire(a_prime(i),ia,time,l,rent1,wage1,H_agg1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%co,egm0%go)
						egm1%co(i,l,ia) = inverse_mu_zero_labor(rhs,hmax0)
						egm1%go(i,l,ia) = egm1%co(i,l,ia) + tfp_gr0*a_prime(i)
					end do
					ss = rep_rate*wage0*H_agg0*shock_fx(l)
					!/* retrieve current financial asset
					asset2(:,l,ia) = (egm1%go(1:na,l,ia)-ss)/(1.0+(1.0-tax_cap0)*rent0) - beq0
				end do
			case(js:jr)
				egm1%cy(0,:,:,:,ia) = 0.0
				egm1%gy(0,:,:,:,ia) = 0.0
				do l = 1,nf
					do k = 1,nt
						do j = 1,nz
							do i = 1,na
								!/* marginal value
								if (ia == jr) then
									rhs = mv_retire(a_prime(i),ia,time,l,rent1,wage1,H_agg1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%co,egm0%go)
								else
									rhs = mv_worker(a_prime(i),ia,time,j,l,rent1,wage1,tax_ss1,tax_lab1,beq1,tfp_gr0,hmax1,tax_cap1,egm0%cy,egm0%gy)
								end if
								egm1%cy(i,j,k,l,ia) = inverse_mu_labor(rhs,ia,j,k,l,wage0,tax_ss0,tax_lab0)
								egm1%wh(i,j,k,l,ia) = labor_from_cons(egm1%cy(i,j,k,l,ia),wage0,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss0,tax_lab0,hmax0)
								if (egm1%wh(i,j,k,l,ia) == 0.0) then
									egm1%cy(i,j,k,l,ia) = inverse_mu_zero_labor(rhs,hmax0)
								end if
								c_tilde = egm1%cy(i,j,k,l,ia) + (1.0-tax_ss0-tax_lab0)*wage0*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*(hmax0-egm1%wh(i,j,k,l,ia))
								egm1%gy(i,j,k,l,ia) = c_tilde + tfp_gr0*a_prime(i)
							end do
							earning = (1.0-tax_ss0-tax_lab0)*wage0*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hmax0
							!/* retrieve current financial asset
							asset1(:,j,k,l,ia) = (egm1%gy(1:na,j,k,l,ia)-earning)/(1.0+(1.0-tax_cap0)*rent0) - beq0
						end do
					end do
				end do
			end select
		end do

		!/* policy function over asset */
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do l = 1,nf
					do k = 1,nt
						do j = 1,nz
							do i = 1,na
								if (asset1(1,j,k,l,ia) > grcap1(i)) then
									policy%y(i,j,k,l,ia) = amin
								else
									policy%y(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),a_prime,grcap1(i))
								end if
								if (policy%y(i,j,k,l,ia) < amin) then
									policy%y(i,j,k,l,ia) = amin
								end if
							end do
							do i = 1,na
								if (asset1(1,j,k,l,ia) > grcap1(i)) then
									cons = interp1(asset1(:,j,k,l,ia),egm1%cy(1:na,j,k,l,ia),grcap1(i))
									policy%l(i,j,k,l,ia) = labor_from_cons(cons,wage0,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss0,tax_lab0,hmax0)
								else
									policy%l(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),egm1%wh(:,j,k,l,ia),grcap1(i))
								end if
								if (policy%l(i,j,k,l,ia) > hmax0) then
									policy%l(i,j,k,l,ia) = hmax0
								elseif (policy%l(i,j,k,l,ia) < 0.0) then
									policy%l(i,j,k,l,ia) = 0.0
								end if
							end do
						end do
					end do
				end do
			case(jr+1:jf-1)
				do l = 1,nf
					do i = 1,na
						if (asset2(1,l,ia) > grcap1(i)) then
							policy%o(i,l,ia) = amin
						else
							policy%o(i,l,ia) = interp1(asset2(:,l,ia),a_prime,grcap1(i))
						end if
						if (policy%o(i,l,ia) < amin) then
							policy%o(i,l,ia) = amin
						end if
					end do
				end do
			case(jf)
				policy%o(:,:,ia) = 0.0
			end select
		end do

	end subroutine end_grid_method_miss

end module mod_end_grid_method