MODULE mod_interpolation
!  Purpose: 
!  Linear and cubic spline interpolation.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   ======================
!  02/13/2005   T. Yamada    From Numerical Recipes
	CONTAINS

	function interp1(x_axis,y_axis,xx)
		use nrtype
		implicit none
		!****** input ******
		real(dp),intent(in),dimension(:) :: x_axis,y_axis
		real(dp),intent(in) :: xx
		!****** output ******
		real(dp) :: interp1
		!****** local variable ******
		real(dp) :: A,B
		integer :: j
		!----------------------------
		j = locate(x_axis,xx)
		if (j == 0) j = 1
		if (j == size(x_axis)) j = size(x_axis) - 1
		if ((x_axis(j+1)-x_axis(j)) == 0.0) then
			A = 0.5
		else
			A = (x_axis(j+1)-xx)/(x_axis(j+1)-x_axis(j))
		end if
		if ((x_axis(j+1)-x_axis(j)) == 0.0) then
			B = 0.5
		else
			B = (xx-x_axis(j))/(x_axis(j+1)-x_axis(j))
		end if
		interp1 = A*y_axis(j) + B*y_axis(j+1)
	end function interp1


	FUNCTION splint(xa,ya,y2a,x)
	USE nrtype
	USE nrutil, ONLY : assert_eq,nrerror
!	USE nr, ONLY: locate
	IMPLICIT NONE
	REAL(DP), DIMENSION(:), INTENT(IN) :: xa,ya,y2a
	REAL(DP), INTENT(IN) :: x
	REAL(DP) :: splint
	INTEGER(I4B) :: khi,klo,n
	REAL(DP) :: a,b,h
	n=assert_eq(SIZE(xa),SIZE(ya),SIZE(y2a),'splint')
	klo=MAX(MIN(locate(xa,x),n-1),1)
	khi=klo+1
	h=xa(khi)-xa(klo)
	IF (h == 0.0) CALL nrerror('bad xa input in splint')
	a=(xa(khi)-x)/h
	b=(x-xa(klo))/h
	splint=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.0_sp
	END FUNCTION splint


	SUBROUTINE spline(x,y,yp1,ypn,y2)
	USE nrtype
	USE nrutil, ONLY : assert_eq
	USE nr, ONLY : tridag
	IMPLICIT NONE
	REAL(DP), DIMENSION(:), INTENT(IN) :: x,y
	REAL(DP), INTENT(IN) :: yp1,ypn
	REAL(DP), DIMENSION(:), INTENT(OUT) :: y2
	INTEGER(I4B) :: n
	REAL(DP), DIMENSION(SIZE(x)) :: a,b,c,r
	n=assert_eq(SIZE(x),SIZE(y),SIZE(y2),'spline')
	c(1:n-1)=x(2:n)-x(1:n-1)
	r(1:n-1)=6.0_sp*((y(2:n)-y(1:n-1))/c(1:n-1))
	r(2:n-1)=r(2:n-1)-r(1:n-2)
	a(2:n-1)=c(1:n-2)
	b(2:n-1)=2.0_sp*(c(2:n-1)+a(2:n-1))
	b(1)=1.0
	b(n)=1.0
	IF (yp1 > 0.99e30_sp) THEN
		r(1)=0.0
		c(1)=0.0
	ELSE
		r(1)=(3.0_sp/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
		c(1)=0.5
	END IF
	IF (ypn > 0.99e30_sp) THEN
		r(n)=0.0
		a(n)=0.0
	ELSE
		r(n)=(-3.0_sp/(x(n)-x(n-1)))*((y(n)-y(n-1))/(x(n)-x(n-1))-ypn)
		a(n)=0.5
	END IF
	CALL tridag(a(2:n),b(1:n),c(1:n-1),r(1:n),y2(1:n))
	END SUBROUTINE spline


	FUNCTION locate(xx,x)
	USE nrtype
	IMPLICIT NONE
	REAL(DP), DIMENSION(:), INTENT(IN) :: xx
	REAL(DP), INTENT(IN) :: x
	INTEGER(I4B) :: locate
	INTEGER(I4B) :: n,jl,jm,ju
	LOGICAL :: ascnd
	n=size(xx)
	ascnd = (xx(n) >= xx(1))
	jl=0
	ju=n+1
	DO
		IF (ju-jl <= 1) EXIT
		jm=(ju+jl)/2
		IF (ascnd .eqv. (x >= xx(jm))) THEN
			jl=jm
		ELSE
			ju=jm
		END IF
	END DO
	IF (x == xx(1)) THEN
		locate=1
	ELSE IF (x == xx(n)) THEN
		locate=n-1
	ELSE
		locate=jl
	END IF
	END FUNCTION locate

END MODULE mod_interpolation


SUBROUTINE tridag_ser(a,b,c,r,u)
USE nrtype; USE nrutil, ONLY : assert_eq,nrerror
IMPLICIT NONE
REAL(DP), DIMENSION(:), INTENT(IN) :: a,b,c,r
REAL(DP), DIMENSION(:), INTENT(OUT) :: u
REAL(DP), DIMENSION(SIZE(b)) :: gam
INTEGER(I4B) :: n,j
REAL(DP) :: bet
n=assert_eq((/SIZE(a)+1,SIZE(b),SIZE(c)+1,SIZE(r),SIZE(u)/),'tridag_ser')
bet=b(1)
IF (bet == 0.0) CALL nrerror('tridag_ser: Error at code stage 1')
u(1)=r(1)/bet
DO j=2,n
	gam(j)=c(j-1)/bet
	bet=b(j)-a(j-1)*gam(j)
	IF (bet == 0.0) &
		CALL nrerror('tridag_ser: Error at code stage 2')
	u(j)=(r(j)-a(j-1)*u(j-1))/bet
END DO
DO j=n-1,1,-1
	u(j)=u(j)-gam(j+1)*u(j+1)
END DO
END SUBROUTINE tridag_ser


RECURSIVE SUBROUTINE tridag_par(a,b,c,r,u)
USE nrtype; USE nrutil, ONLY : assert_eq,nrerror
USE nr, ONLY : tridag_ser
IMPLICIT NONE
REAL(DP), DIMENSION(:), INTENT(IN) :: a,b,c,r
REAL(DP), DIMENSION(:), INTENT(OUT) :: u
INTEGER(I4B), PARAMETER :: NPAR_TRIDAG=4
INTEGER(I4B) :: n,n2,nm,nx
REAL(DP), DIMENSION(SIZE(b)/2) :: y,q,piva
REAL(DP), DIMENSION(SIZE(b)/2-1) :: x,z
REAL(DP), DIMENSION(SIZE(a)/2) :: pivc
n=assert_eq((/SIZE(a)+1,SIZE(b),SIZE(c)+1,SIZE(r),SIZE(u)/),'tridag_par')
IF (n < NPAR_TRIDAG) THEN
	CALL tridag_ser(a,b,c,r,u)
ELSE
	IF (MAXVAL(ABS(b(1:n))) == 0.0) &
		CALL nrerror('tridag_par: possible singular matrix')
	n2=SIZE(y)
	nm=SIZE(pivc)
	nx=SIZE(x)
	piva = a(1:n-1:2)/b(1:n-1:2)
	pivc = c(2:n-1:2)/b(3:n:2)
	y(1:nm) = b(2:n-1:2)-piva(1:nm)*c(1:n-2:2)-pivc*a(2:n-1:2)
	q(1:nm) = r(2:n-1:2)-piva(1:nm)*r(1:n-2:2)-pivc*r(3:n:2)
	IF (nm < n2) THEN
		y(n2) = b(n)-piva(n2)*c(n-1)
		q(n2) = r(n)-piva(n2)*r(n-1)
	END IF
	x = -piva(2:n2)*a(2:n-2:2)
	z = -pivc(1:nx)*c(3:n-1:2)
	CALL tridag_par(x,y,z,q,u(2:n:2))
	u(1) = (r(1)-c(1)*u(2))/b(1)
	u(3:n-1:2) = (r(3:n-1:2)-a(2:n-2:2)*u(2:n-2:2) &
		-c(3:n-1:2)*u(4:n:2))/b(3:n-1:2)
	IF (nm == n2) u(n)=(r(n)-a(n-1)*u(n-1))/b(n)
END IF
END SUBROUTINE tridag_par