subroutine transition
!  Purpose
!  Compute a transition path between initial and final steady states.
!
!  Record of Revisions:
!    Date      Programmer   Description of Change
!  ==========   ==========   =====================
!  02/02/2011   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_interpolation
	use mod_preamble
	use mod_end_grid_method
	use mod_law_of_motion
	implicit none

	!****** local variables ******
	integer :: it_main,it,i
	real(dp) :: adj = 0.2
	real(dp) :: inc
	real(dp) :: metric=1.0
	real(dp) :: metric_cap=1.0
	real(dp) :: metric_lab=1.0
	real(dp) :: metric_rent=1.0
	real(dp) :: metric_wage=1.0
	integer,parameter :: maxit=100
	real(dp),parameter :: tol=1.0e-004

	!****** macro variables ******
	real(dp) :: KoverH,KoverY
	real(dp) :: K_supply,H_supply

	!****** policy functions ******
	type(policy_egm) :: egm0,egm1,egm_miss(1980:2001)

	!****** age-capital profile ******
	real(dp),dimension(js:jf,ts:tf) :: K_age
	real(dp),dimension(js:jf) :: K_mean_age,K_next_age

	!****** age-labor profile ******
	real(dp),dimension(js:jr,ts:tf) :: H_age
	real(dp),dimension(js:jr) :: H_mean_age,H_hour_age

	!****** density function ******
	type(Phi_function) :: phi,phi_new
	!------------------------------

	write(*,*) "Start to compute a transition path"
	write(*,*) ""

	!****** main loop ******
	open(10,file='metric.txt')
	do it_main = 1,maxit

		!/* iteration error check
		if (metric < tol) exit
		write(*,*) "iteration index,", it_main
		write(*,"(' metric: capital,', f10.4,'%')") metric_cap*100.0
		write(*,"(' metric: labor,  ', f10.4,'%')") metric_lab*100.0
		write(*,"(' metric: rent,   ', f10.4,'%')") metric_rent*100.0
		write(*,"(' metric: wage,   ', f10.4,'%')") metric_wage*100.0

		!/* set final steady state's policy function
		egm0 = egm_fin

		!/* household's optimization
		write(*,*) "-+-+- Now, computing policy function -+-+-"
		do it = tf,ts,-1
			if (it > 2001) then
				!/* true policy
				call end_grid_method_tr(it,egm0,egm1,policy)
				policy_pre(it) = policy
			elseif (it == 2001) then
				!/* mis-predicted policy
				call end_grid_method_pre(it,egm0,egm_miss(it),policy_pre(it))
			elseif (it <= 2000) then
				!/* mis-predicted policy
				call end_grid_method_miss(it,egm_miss(it+1),egm_miss(it),policy)
				!/* realized policy
				call end_grid_method_pre(it,egm_miss(it+1),egm1,policy_pre(it))
			end if
			!/* update egm: this function is "perfect foresight"
			egm0 = egm1
		end do

		!/* distribution for each year
		phi = phi_ini
		write(*,*) "-+-+- Now, computing density function -+-+-"
		write(*,*) ""
		do it = ts,tf

			!/* transition law of motion
			call law_of_motion_tr(it,policy_pre(it),state0(it),phi,K_mean_age,K_next_age,H_mean_age,H_hour_age,phi_new)

			!/* update density function
			phi = phi_new

			!/* aggregate variables
			do i = js,jf 
				K_age(i,it) = mu(i,it)*K_mean_age(i)
			end do
			do i = js,jr
				H_age(i,it) = mu(i,it)*H_mean_age(i)
			end do
			K_supply = sum(K_age(:,it))
			H_supply = sum(H_age(:,it))

			!/* update
			state1(it)%tfp_gr  = state0(it)%tfp_gr
			state1(it)%hmax    = state0(it)%hmax
			state1(it)%theta   = state0(it)%theta
			state1(it)%delta   = state0(it)%delta
			state1(it)%tax_cap = state0(it)%tax_cap
			state1(it)%tax_ss  = state0(it)%tax_ss
			state1(it)%goy     = state0(it)%goy

			!/* new capital, labor path and accidental bequest
			state1(it)%cap = adj*K_supply + (1.0-adj)*state0(it)%cap
			state1(it)%lab = adj*H_supply + (1.0-adj)*state0(it)%lab
			KoverH = state1(it)%cap/state1(it)%lab
			call factor_price(KoverH,tfp,state1(it)%theta,state1(it)%delta,state1(it)%rent,state1(it)%wage,KoverY)
			state1(it)%beq = 0.0
			do i = js,jf
				state1(it)%beq = (1.0-surv(i,it))*mu(i,it)*K_next_age(i) + state1(it)%beq
			end do

			!/* new G/Y
			state1(it)%gov_exp = state1(it)%goy*(tfp*(state1(it)%cap**state1(it)%theta)*(state1(it)%lab**(1.0-state1(it)%theta)))
			call govt_budget_general(state1(it)%gov_exp,state1(it)%rent,state1(it)%wage,state1(it)%cap,state1(it)%lab,state1(it)%tax_cap,state1(it)%tax_lab)

		end do

		!/* iteration error
		metric_cap  = maxval(abs((state1(ts+1:tf-1)%cap-state0(ts+1:tf-1)%cap) / state0(ts+1:tf-1)%cap))
		metric_lab  = maxval(abs((state1(ts+1:tf-1)%lab-state0(ts+1:tf-1)%lab) / state0(ts+1:tf-1)%lab))
		metric_rent = maxval(abs((state1(ts+1:tf-1)%rent-state0(ts+1:tf-1)%rent) / state0(ts+1:tf-1)%rent))
		metric_wage = maxval(abs((state1(ts+1:tf-1)%wage-state0(ts+1:tf-1)%wage) / state0(ts+1:tf-1)%wage))
		metric      = max(metric_cap,metric_lab,metric_rent,metric_wage)

		!/* output tentative results
		write(10,*) "iteration index,", it_main
		write(10,*) "metric: capital path,", metric_cap
		write(10,*) "metric: labor path,  ", metric_lab
		write(10,*) "metric: rent path,   ", metric_rent
		write(10,*) "metric: wage path,   ", metric_wage
		write(10,*) ""
		call write_equilibrium_path

		!/* update for new path
		state0 = state1

		!/* initial and final state is fixed
		state0(ts) = state_ini
		state0(tf) = state_fin

	end do

	!/* economic inequality */
	call compute_inequality

	!/* computation time */
	call cpu_time(time_end)
	write(10,*) "Program Finished Sucessfully in", time_end-time_begin, "seconds"
	write(10,*) "The Number of Total Iteration", it_main
	close(10)

end subroutine transition