module mod_preamble
!  Purpose:
!  Collects several functions that is used in economics.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  02/02/2011   T. Yamada    Collect subroutines
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none	
	contains

	subroutine set_initial
	!  Purpose:
	!  Set initial TFP, capital share, depreciation rate
	!  income tax rate, and maximum of time endowment. 
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  11/24/2010   T. Yamada    Original code
		implicit none
		!/* set exogenous variables
		select case (flag_year)
		case(1)
			tfp     = tfp80
			tfp_gr  = tfp_gr80
			hmax    = hmax80
			theta   = theta80
			delta   = delta80
			tax_cap = tax80
			goy     = goy80
		case(2)
			tfp     = tfp22
			tfp_gr  = tfp_gr22
			hmax    = hmax00
			theta   = theta22
			delta   = delta22
			tax_cap = tax22
			goy     = goy22
		case(3)
			tfp     = tfp70s
			tfp_gr  = tfp_gr70s
			hmax    = hmax80
			theta   = theta70s
			delta   = delta70s
			tax_cap = tax70s
			goy     = goy70s
		case(4)
			tfp     = tfp80s
			tfp_gr  = tfp_gr80s
			hmax    = hmax80
			theta   = theta80s
			delta   = delta80s
			tax_cap = tax80s
			goy     = goy80s
		case(5)
			tfp     = tfp90s
			tfp_gr  = tfp_gr90s
			hmax    = hmax00
			theta   = theta90s
			delta   = delta90s
			tax_cap = tax90s
			goy     = goy90s
		case(6:)
			!/* for counterfactual simulation [Hayashi&Prescott]
			tfp     = tfp80
			tfp_gr  = 1.020
			hmax    = 16.0*5.5*4.0*12.0
			theta   = 0.362
			delta   = 0.089
			tax_cap = 0.480
			goy     = 0.150
		end select
	end subroutine set_initial


	subroutine wage_risk
	!  Purpose:
	!  Wage risk consists of three components.
	!  (1) fixed effect
	!  (2) persistent shock
	!  (3) transitory shock
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  08/01/2007   T. Yamada    Original code
		use mod_Markov,only:Tauchen1986
		implicit none
		!****** local variables ******
		integer :: i
		real(dp) :: var_eps,std_z,mm
		real(dp),dimension(js:jr) :: std_eps
		real(dp),dimension(nz) :: pi_dist,dummy
		!-----------------------------
		!/* hourly wage
		open(11,file='real_wage_90_00.in')
			read(11,*) kappa
		close(11)
		!/* var and sd of wage at 20
		std_eps(js) = std_eps20
		var_eps = std_eps(js)**2
		std_z   = sqrt(var_eps/(1.0-(rho**2)))
		!/* Tauchen(1986)'s approximation
		mm = 1.2*log(real(nz))
		call Tauchen1986(nz,rho,mu_z,std_eps(js),std_z,mm,prob,pi_dist,dummy,endow(:,js))
		!/* persistent shock by age */
		do i = js+1,jr
			!/* linear income variance profile
			std_eps(i) = std_eps(i-1) + std_pr
			var_eps    = std_eps(i)**2
			std_z      = sqrt(var_eps/(1.0-(rho**2)))
			!/* Tauchen (1986,EL)'s approximation
			call Tauchen1986(nz,rho,mu_z,std_eps(i),std_z,mm,prob,pi_dist,dummy,endow(:,i))
		end do
		!/* transitory shock */
		if (nt == 1) then
			shock_tr(1) = 1.0
			prob_tr(1)  = 1.0
		elseif (nt == 2) then
			shock_tr(1) = exp(-sig_tr)
			shock_tr(2) = 1.0+(1.0-exp(-sig_tr))
			prob_tr(:)  = 0.5
		end if
		!/* permanent shock */
		if (nf == 1) then
			shock_fx(1) = 1.0
			prob_fx(1)  = 1.0
		elseif (nf == 2) then
			shock_fx(1) = exp(-sig_fx)
			shock_fx(2) = 1.0+(1.0-exp(-sig_fx))
			prob_fx(:)  = 0.5
		end if
	end subroutine wage_risk


	subroutine factor_price(KoverL,tfp,share,depre,rent,wage,KoverY)
	!  Purpose:
	!  Assume that the production function is of Cobb-Douglas type.
	!  Compute the interest rate and the wage given K over L.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  11/05/2006   Tomoaki Y.   Original code
		implicit none
		!****** input ******
		real(dp),intent(in)  :: KoverL,tfp,share,depre
		!****** output ******
		real(dp),intent(out) :: rent,wage,KoverY
		!--------------------
		rent   =      share *tfp*(KoverL**(share-1.0)) - depre
		wage   = (1.0-share)*tfp*(KoverL**share)
		KoverY = KoverL**(1.0-share)
	end subroutine factor_price


	subroutine govt_budget_ss(frac_ret,tax_ss)
	!  Purpose:
	!  The government's budget constraint.
	!  Given a replacement rate, compute an equilibrium payroll tax rate.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  11/08/2006   T. Yamada    Original code
	!  02/02/2010   T. Yamada    Remove wage and H_agg
		implicit none
		!****** input ******
		real(dp),intent(in) :: frac_ret
		!****** output ******
		real(dp),intent(out) :: tax_ss
		!--------------------
		tax_ss = rep_rate*frac_ret
	end subroutine govt_budget_ss


	subroutine govt_budget_general(gov_exp,rent,wage,K_agg,H_agg,tax_lab)
	!  Purpose:
	!  The government's budget constraint.
	!  From exogenously given government expenditure
	!  and revenue from capital income tax,
	!  compute labor income tax rate.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  02/08/2011   T. Yamada    Original code
		implicit none
		!****** input ******
		real(dp),intent(in) :: gov_exp,rent,wage
		real(dp),intent(in) :: K_agg,H_agg
		!****** output ******
		real(dp),intent(out) :: tax_lab
		!****** local variables ******
		real(dp) :: revenue
		!-----------------------------
		revenue = tax_cap*rent*K_agg
		tax_lab = (gov_exp-revenue)/(wage*H_agg)
	end subroutine govt_budget_general

end module mod_preamble