subroutine demographics
!  Purpose:
!  Load survival probabilities and calculate demographic structure.
!
!  Data Source:
!  the National Institute of Population and Social Security Research
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  01/26/2007   T. Yamada    Original code
!  11/20/2007   T, Yamada    Up-dated to h18 version
!  01/25/2009   T, Yamada    Revised to Submit JEDC
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none

	!****** local variables ******
	integer :: i,j,status
	real(dp),allocatable,dimension(:) :: surv_all
	real(dp),allocatable,dimension(:,:) :: surv_1980_2055,mu_tran
	real(dp),dimension(ts:tt-1) :: pop_growth_1980_2054
	real(dp),dimension(tt:2065) :: pop_growth_2055_2065
	real(dp),dimension(0:jf) :: mu_1980,nu
	real(dp),dimension(0:jf,0:jf) :: GAM
	!-----------------------------

	allocate(surv_all(jf*(tt-ts+1)),STAT=status)
	allocate(surv_1980_2055(0:jf,ts:tt),STAT=status)
	allocate(mu_tran(0:jf,ts:tf),STAT=status)
	surv_all       = 0.0
	surv_1980_2055 = 0.0
	mu_tran        = 0.0

	!/* survival probabilities from 1980 to 2055
	open(10,file='surv_1980_2055.in')
		read(10,*) surv_all
	close(10)
	do i = ts,tt
		surv_1980_2055(0:jf-1,i) = surv_all((i-ts)*jf+1:jf*(i-ts+1))
	end do
	surv_1980_2055(jf,:) = 0.0

	!/* population dynamics
	select case (flag_pop)
	case(1)

		!/* use actual population distribution
		open(11,file='pop_1980.in')
			read(11,*) mu_1980
		close(11)
		mu_tran(:,ts) = mu_1980

		!/* fertility rate from 1980 to 2054: use medium variant
		open(12,file='pop_growth_1980_2054_medium.in')
			read(12,*) pop_growth_1980_2054
		close(12)

		!/* linear convergence of population growth rate from 2055 to 2065
		pop_growth_2055_2065(2065) = 1.0
		do i = 2055,2064,1
			pop_growth_2055_2065(i) = pop_growth_1980_2054(2054) + ((pop_growth_2055_2065(2065)-pop_growth_1980_2054(2054))/real(2064-2054))*(i-2054)
		end do

		!/* population dynamics from 1980 to 2200
		do i = ts,tf-1
			GAM = 0.0
			select case(i)
			!/* 1980-2054
			case(1:tt-1)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,i)
				end do
				GAM(0,0)       = pop_growth_1980_2054(i)
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			!/* 2055-2065
			case(tt:2065)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,tt)
				end do
				GAM(0,0)       = pop_growth_2055_2065(i)
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			!/* 2066-2200
			case(2066:)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,tt)
				end do
				GAM(0,0)       = 1.0
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			end select
		end do

	case(2:)

		!/* use survival probability in 2000
		do i = ts,tt
			surv_1980_2055(:,i) = surv_1980_2055(:,tc)
		end do

		!/* initial population distribution
		nu(0) = 1.0
		do j = 0,jf-1
			nu(j+1) = surv_1980_2055(j,tc)/pop_gr*nu(j)
		end do
		do j = 0,jf
			mu_tran(j,ts) = nu(j)/sum(nu(0:jf))
		end do

		!/* population dynamics
		do i = ts,tf-1
			GAM = 0.0
			select case(i)
			!/* 1980-2054
			case(ts:tt-1)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,i)
				end do
				GAM(0,0)       = pop_gr
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			!/* 2055-2065
			case(tt:2065)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,tt)
				end do
				GAM(0,0)       = pop_gr
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			!/* 2066-2200
			case(2066:tf)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,tt)
				end do
				GAM(0,0)       = pop_gr
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			end select
		end do

	end select

	!/* set initial/final population distribution
	select case(flag_year)
	case(1)
		!/* initial state: 1980
		surv     = surv_1980_2055(js:jf,ts)
		mu       = mu_tran(js:jf,ts)/sum(mu_tran(js:jf,ts))
		frac_ret = sum(mu(jr+1:jf))
	case(2)
		!/* final state: 2200
		surv     = surv_1980_2055(js:jf,tt)
		mu       = mu_tran(js:jf,tf)/sum(mu_tran(js:jf,tf))
		frac_ret = sum(mu(jr+1:jf))
	case(3)
		surv     = surv_1980_2055(js:jf,1980)
		mu       = mu_tran(js:jf,1980)/sum(mu_tran(js:jf,1980))
		frac_ret = sum(mu(jr+1:jf))
	case(4)
		surv     = surv_1980_2055(js:jf,1990)
		mu       = mu_tran(js:jf,1990)/sum(mu_tran(js:jf,1990))
		frac_ret = sum(mu(jr+1:jf))
	case(5)
		surv     = surv_1980_2055(js:jf,2000)
		mu       = mu_tran(js:jf,2000)/sum(mu_tran(js:jf,2000))
		frac_ret = sum(mu(jr+1:jf))
	case(6:)
		!/* benchmark: Hayashi and Prescott (2002)
		surv     = surv_1980_2055(js:jf,tc)
		mu       = mu_tran(js:jf,tc)/sum(mu_tran(js:jf,tc))
		frac_ret = sum(mu(jr+1:jf))
	end select

	deallocate(surv_all)
	deallocate(surv_1980_2055)
	deallocate(mu_tran)

end subroutine demographics