subroutine steady_state
!  Purpose:
!  Compute a steady state of a lifecyle model.
!
!  Record of Revisions:
!     Date      Programmer   Description of Change
!  ==========   ==========   =====================
!  0//02/2011   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_preamble
	implicit none

	!****** main loop ******
	integer :: it_gy,it_kl,i,j
	real(dp) :: metric_cap = 1.0
	real(dp) :: metric_lab = 1.0
	real(dp) :: metric_gy = 1.0
	integer,parameter :: maxit = 1000
	real(dp),parameter :: adj = 0.2
	real(dp),parameter :: tol_cap=1.0e-003
	real(dp),parameter :: tol_lab=1.0e-003
	real(dp),parameter :: tol_gy=1.0e-003
		
	!****** policy functions ******
	type(policy_function) :: policy

	!****** macro variables ******
	real(dp) :: K_agg,K_supply
	real(dp) :: H_agg,H_supply
	real(dp) :: KoverH,KoverY
	real(dp) :: Y_agg

	!****** factor prices ******
	real(dp) :: wage,rent

	!****** tax rates ******
	real(dp) :: tax_ss
	real(dp) :: tax_lab
	real(dp) :: gov_exp,gov_exp_new

	!****** accidental bequest ******
	real(dp) :: beq
	
	!****** age-capital profile ******
	real(dp),dimension(js:jf) :: K_age,K_mean_age,K_next_age

	!****** age-labor profile ******
	real(dp),dimension(js:jr) :: H_age,H_mean_age,H_hour_age

	!****** density function ******
	type(phi_function) :: phi
	!------------------------------

	!/* initial guess of capital and labor */
	K_agg = 1500.0
	H_agg =  500.0
	beq = 10.0
	KoverH = K_agg/H_agg
	write(*,"(' Initial Guess of K/H:    ',f10.4)") KoverH

	!/* initial factor prices & payroll tax rate */
	call factor_price(KoverH,tfp,theta,delta,rent,wage,KoverY)
	call govt_budget_ss(frac_ret,tax_ss)

	write(*,*) ""
	write(*,*) "-+-+-+ Payroll Tax Rate +-+-+-"
	write(*,"(' Payroll Tax Rate,        ', f10.4,'%')") tax_ss*100.0

	!/* government expenditure */
	Y_agg = tfp*(K_agg**theta)*(H_agg**(1.0-theta))
	gov_exp = goy*Y_agg
	tax_lab = 0.10

	!****** main loop ******
	do it_gy = 1,maxit

		if (metric_gy<tol_gy) exit
		metric_cap = 1.0
		metric_lab = 1.0

		write(*,*) ""
		write(*,*) "-+-+-+-+- iteration counter (G/Y) -+-+-+-+-", it_gy
		write(*,"(' iteration error: G/Y,    ', f10.4,'%')") metric_gy*100.0

		do it_kl = 1,maxit

			if (metric_cap<tol_cap .and. metric_lab<tol_lab) exit

			write(*,*) ""
			write(*,*) "iteration counter:                ", it_kl
			write(*,"(' after-tax interest rate: ', f10.4,'%')") (1.0-tax_cap)*rent*100.0
			write(*,"(' wage:                    ', f10.4)") wage

			!/* household's optimizaiton
			call end_grid_method(rent,wage,tax_ss,tax_lab,H_agg,beq,policy)

			!/* stationary distribution and profiles
			call law_of_motion(policy,phi,K_mean_age,K_next_age,H_mean_age,H_hour_age)

			!/* aggregation
			do i = js,jf
				K_age(i) = mu(i)*K_mean_age(i)
			end do
			do i = js,jr
				H_age(i) = mu(i)*H_mean_age(i)
			end do
			K_supply = sum(K_age)
			H_supply = sum(H_age)
			write(*,"(' aggregate capital supply:', f10.4)") K_supply
			write(*,"(' aggregate labor supply:  ', f10.4)") H_supply

			!/* accidental bequest
			beq = 0.0
			do i = js,jf
				beq = (1.0-surv(i))*mu(i)*K_next_age(i) + beq
			end do

			!/* iteration error
			metric_cap = abs((K_supply-K_agg)/K_supply)
			metric_lab = abs((H_supply-H_agg)/H_supply)
			write(*,"(' iteration error: capital,', f10.4,'%')") metric_cap*100.0
			write(*,"(' iteration error: labor,  ', f10.4,'%')") metric_lab*100.0

			!/* up-data & new factor prices
			K_agg = adj*K_supply + (1.0-adj)*K_agg
			H_agg = adj*H_supply + (1.0-adj)*H_agg
			KoverH = K_agg/H_agg
			call factor_price(KoverH,tfp,theta,delta,rent,wage,KoverY)

		end do

		Y_agg = tfp*(K_agg**theta)*(H_agg**(1.0-theta))
		gov_exp_new = goy*Y_agg
		call govt_budget_general(gov_exp_new,rent,wage,K_agg,H_agg,tax_lab)
		metric_gy = abs((gov_exp-gov_exp_new)/gov_exp)
		gov_exp = gov_exp_new

	end do

	!/* results */
	call all_results(policy,phi,rent,wage,tax_ss,tax_lab,K_agg,H_agg,beq,gov_exp,K_mean_age,H_mean_age,H_hour_age)

end subroutine steady_state