module mod_generate_grid
!  Purpose:
!  Generate meshed grids (evaluation points).
!  The grids are spaced uniformly or exponential.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   ======================
!  10/21/2010   T. Yamada    Current version
	implicit none
	contains

	subroutine grid_uniform(mink,maxk,num_grid,grid)
	!  Purpose: 
	!  Generate a uniform grid between [maxk, mink].
	!  The number of grids is num_grid.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/19/2003   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		integer :: i
		real(dp) :: increment
		!-----------------------------
		increment = (maxk-mink)/real(num_grid-1)
		do i = 1,num_grid
			grid(i) = (i-1)*increment+mink
		end do
		if (grid(num_grid)/=maxk) then
			grid(num_grid) = maxk
		endif
	end subroutine grid_uniform


	subroutine grid_exp(mink,maxk,num_grid,grid)
	!  Purpose:
	!  Generatea an exponential grid.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  10/21/2010   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		real(dp) :: maxd
		real(dp),dimension(num_grid) :: mesh
		!-----------------------------
		maxd = log(maxk+1.0)
		call grid_uniform(mink,maxd,num_grid,mesh)
		grid = exp(mesh)-1
	end subroutine grid_exp


	subroutine grid_double_exp(mink,maxk,num_grid,grid)
	!  Purpose:
	!  Generatea a double exponential grid.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  10/21/2010   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		real(dp) :: maxd
		real(dp),dimension(num_grid) :: mesh
		!-----------------------------
		maxd = log(log(maxk+1.0)+1.0)
		call grid_uniform(mink,maxd,num_grid,mesh)
		grid = exp(exp(mesh)-1)-1
	end subroutine grid_double_exp


	subroutine grid_triple_exp(mink,maxk,num_grid,grid)
	!  Purpose:
	!  Generatea a triple exponential grid.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  10/21/2010   T. Yamada    Original code
		use nrtype,only:dp
		!****** input ******
		real(dp),intent(in) :: maxk,mink
		integer,intent(in) :: num_grid
		!****** output ******
		real(dp),intent(out),dimension(num_grid) :: grid
		!****** local variables ******
		real(dp) :: maxd
		real(dp),dimension(num_grid) :: mesh
		!-----------------------------
		maxd = log(log(log(maxk+1.0)+1.0)+1.0)
		call grid_uniform(mink,maxd,num_grid,mesh)
		grid = exp(exp(exp(mesh)-1)-1)-1
	end subroutine grid_triple_exp

end module mod_generate_grid