module mod_preamble
!  Purpose:
!  Collects several subroutines and functions.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  07/19/2003   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none	
	contains

	subroutine wage_risk
	!  Purpose:
	!  Wage risk consists of three components.
	!  (1) fixed effect
	!  (2) persistent shock
	!  (3) transitory shock
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  08/01/2007   T. Yamada    Extended for 64bit
	!  03/30/2008   T. Yamada    Revised to submit JEDC
		use mod_Markov,only:Tauchen1986
		implicit none
		!****** local variables ******
		integer :: i
		real(dp) :: var_eps,std_z,w,mm
		real(dp),dimension(js:jr) :: std_eps
		real(dp),dimension(nz) :: pi_dist,dummy
		!-----------------------------
		!/* hourly wage */
		open(10,file='real_wage_90_00.in')
			read(10,*) kappa
		close(10)
		!/* var and std of wage at 20
		std_eps(js) = std_eps20
		var_eps = std_eps(js)**2
		std_z   = sqrt(var_eps/(1.0-(rho**2)))
		!/* Tauchen(1986,EL) based approximation
		mm = 1.2*log(real(nz))
		call Tauchen1986(nz,rho,mu_z,std_eps(js),std_z,mm,prob,pi_dist,dummy,endow(:,js))
		!/* persistent shock */
		do i = js+1,jr
			!/* linear income variance profile
			std_eps(i) = std_eps(i-1) + std_pr
			var_eps    = std_eps(i)**2
			std_z      = sqrt(var_eps/(1.0-(rho**2)))
			!/* Tauchen(1986,EL) based approximation
			call Tauchen1986(nz,rho,mu_z,std_eps(i),std_z,mm,prob,pi_dist,dummy,endow(:,i))
		end do
		!/* transitory shock */
		if (nt == 1) then
			shock_tr(1) = 1.0
			prob_tr(1)  = 1.0
		elseif (nt == 2) then
			shock_tr(1) = exp(-sig_tr)
			shock_tr(2) = 1.0+(1.0-exp(-sig_tr))
			prob_tr(:)  = 0.5
		end if
		!/* permanent shock */
		if (nf == 1) then
			shock_fx(1) = 1.0
			prob_fx(1)  = 1.0
		elseif (nf == 2) then
			shock_fx(1) = exp(-sig_fx)
			shock_fx(2) = 1.0+(1.0-exp(-sig_fx))
			prob_fx(:)  = 0.5
		end if
	end subroutine wage_risk


	subroutine factor_price(KoverL,tfp,share,depre,rent,wage,KoverY)
	!  Purpose
	!  Assuming that the production function is of Cobb-Douglas type,
	!  compute the interest rate and the wage given K over L.
	!
	!  Input:
	!    KoverL  :capital over labor supply
	!    tfp     :total factor productivity
	!    share   :capital share
	!    depre   :depreciation rate
	!
	!  Output:
	!    rent    :interest rate
	!    wage    :wage
	!    KoverY  :capital over GDP
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  11/05/2006   T. Yamada    Original code
		implicit none
		!****** input ******
		real(dp),intent(in)  :: KoverL,tfp,share,depre
		!****** output ******
		real(dp),intent(out) :: rent,wage,KoverY
		!--------------------
		rent   =      share *tfp*(KoverL**(share-1.0)) - depre
		wage   = (1.0-share)*tfp*(KoverL**share)
		KoverY = KoverL**(1.0-share)
	end subroutine factor_price


	subroutine govt_budget_ss(frac_ret,tax_ss)
	!  Purpose:
	!  The government's budget constraint for social security system.
	!  Given replacement rate, compute an equilibrium payroll tax rate.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  11/08/2006   T. Yamada    Original code
		implicit none
		!****** input ******
		real(dp),intent(in) :: frac_ret
		!****** output ******
		real(dp),intent(out) :: tax_ss
		!-------------------
		tax_ss = rep_rate*frac_ret
	end subroutine govt_budget_ss


	subroutine govt_budget_general(gov_exp,rent,wage,K_agg,H_agg,tax_cap,tax_lab)
	!  Purpose:
	!  The government's budget constraint.
	!  From exogenously given government expenditure
	!  and revenue from capital income tax,
	!  compute labor income tax rate.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  02/08/2011   T. Yamada    Original code
		implicit none
		!****** input ******
		real(dp),intent(in) :: gov_exp,rent,wage
		real(dp),intent(in) :: K_agg,H_agg,tax_cap
		!****** output ******
		real(dp),intent(out) :: tax_lab
		!****** local variables ******
		real(dp) :: revenue
		!-----------------------------
		revenue = tax_cap*rent*K_agg
		tax_lab = (gov_exp-revenue)/(wage*H_agg)
	end subroutine govt_budget_general


	subroutine set_exogenous_path
	!  Purpose:
	!  Set exogenous paths from external text files.
	!  (1) TFP factor growth rate
	!  (2) available time
	!  (3) capital share
	!  (4) depreciation rate
	!  (5) capital income tax rate
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  02/02/2011   T. Yamada    Original code
		implicit none
		!****** local variables ******
		integer :: i
		!-----------------------------
		!/* TFP factor growth rate
		select case (flag_tfp)
		case(1)
			select case(flag_theta)
			case(1)
				open(30,file='adjusted_TFP_growth_1980_2000.in')
			case(2)
				open(30,file='TFP_growth_1980_2000.in')
			end select
				read(30,*) state0(1980:1999)%tfp_gr
			close(30)
			!Notice[data on TFP is btwn 1980-1999]
			do i = 2000,2010
				state0(i)%tfp_gr = state0(1999)%tfp_gr + (tfp_gr_fin-state0(1999)%tfp_gr)/11.0*real(i-1999)
			end do
			state0(2011:2200)%tfp_gr = tfp_gr_fin
		case(2:)
			state0(1980:2000)%tfp_gr = tfp_gr_con
			do i = 2001,2010
				state0(i)%tfp_gr = state0(2000)%tfp_gr + (tfp_gr_fin-state0(2000)%tfp_gr)/10.0*real(i-2000)
			end do
			state0(2011:2200)%tfp_gr = tfp_gr_fin
		end select
		state_ini%tfp_gr = state0(ts)%tfp_gr
		state_fin%tfp_gr = state0(tf)%tfp_gr
		!/* time endowment
		select case (flag_time)
		case(1)
			state0(1980:1988)%hmax = 16.0*5.5*4.0*12.0
			state0(1993:2200)%hmax = 16.0*5.0*4.0*12.0
			do i = 1989,1992
				state0(i)%hmax = state0(1988)%hmax + (state0(1993)%hmax-state0(1988)%hmax)/5.0*real(i-1988)
			end do
		case(2:)
			state0(:)%hmax = 16.0*5.5*4.0*12.0
		end select
		state_ini%hmax = state0(ts)%hmax
		state_fin%hmax = state0(tf)%hmax
		!/* capital share
		select case (flag_theta)
		case(1)
			open(31,file='share_1980_2000.in')
				read(31,*) state0(1980:2000)%theta
			close(31)
			do i = 2001,2010
				state0(i)%theta = state0(2000)%theta + (theta_fin-state0(2000)%theta)/10.0*real(i-2000)
			end do
			state0(2011:2200)%theta = theta_fin
		case(2:)
			state0(1980:2000)%theta = theta_con
			do i = 2001,2010
				state0(i)%theta = state0(2000)%theta + (theta_fin-state0(2000)%theta)/10.0*real(i-2000)
			end do
			state0(2011:2200)%theta = theta_fin
		end select
		state_ini%theta = state0(ts)%theta
		state_fin%theta = state0(tf)%theta
		!/* depreciation rate
		select case (flag_depre)
		case(1)
			open(32,file='depreciation_1980_2000.in')
				read(32,*) state0(1980:2000)%delta
			close(32)
			do i = 2001,2010
				state0(i)%delta = state0(2000)%delta + (delta_fin-state0(2000)%delta)/10.0*real(i-2000)
			end do
			state0(2011:2200)%delta = delta_fin
		case(2:)
			state0(1980:2000)%delta = delta_con
			do i = 2001,2010
				state0(i)%delta = state0(2000)%delta + (delta_fin-state0(2000)%delta)/10.0*real(i-2000)
			end do
			state0(2011:2200)%delta = delta_fin
		end select
		state_ini%delta = state0(ts)%delta
		state_fin%delta = state0(tf)%delta
		!/* capital income tax rate
		select case (flag_tax)
		case(1)
			open(32,file='cap_tax_1980_2000.in')
				read(32,*) state0(1980:2000)%tax_cap
			close(32)
			do i = 2001,2010
				state0(i)%tax_cap = state0(2000)%tax_cap + (tax_cap_fin-state0(2000)%tax_cap)/10.0*real(i-2000)
			end do
			state0(2011:2200)%tax_cap = tax_cap_fin
		case(2:)
			state0(1980:2000)%tax_cap = tax_cap_con
			do i = 2001,2010
				state0(i)%tax_cap = state0(2000)%tax_cap + (tax_cap_fin-state0(2000)%tax_cap)/10.0*real(i-2000)
			end do
			state0(2011:2200)%tax_cap = tax_cap_fin
		end select
		state_ini%tax_cap = state0(ts)%tax_cap
		state_fin%tax_cap = state0(tf)%tax_cap
		!/* G/Y
		select case (flag_gy)
		case(1)
			open(33,file='g_over_y_1980_2000.in')
				read(33,*) state0(1980:2000)%goy
			close(33)
			do i = 2001,2010
				state0(i)%goy = state0(2000)%goy + (goy_fin-state0(2000)%goy)/10.0*real(i-2000)
			end do
			state0(2011:2200)%goy = goy_fin
		case(2)
			state0(1980:2000)%goy = goy_con
			do i = 2001,2010
				state0(i)%goy = state0(2000)%goy + (goy_fin-state0(2000)%goy)/10.0*real(i-2000)
			end do
			state0(2011:2200)%goy = goy_fin
		end select
		state_ini%goy = state0(ts)%goy
		state_fin%goy = state0(tf)%goy
	end subroutine set_exogenous_path


	subroutine set_initial_path
	!  Purpose:
	!  Guess initial paths.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  01/21/2007   T. Yamada    Original code
	!  11/08/2008   T. Yamada    Revised to submit JEDC
		implicit none
		!****** local variables ******
		integer :: i
		real(dp) :: inc,KoverH,KoverY
		!-----------------------------
		state0(ts) = state_ini
		state0(tf) = state_fin
		!/* capital path
		inc = (state_fin%cap-state_ini%cap)/real(tf-ts-1)
		do i = ts+1,tf-1
			state0(i)%cap = state0(i-1)%cap + inc
		end do
		!/* labor path
		inc = (state_fin%lab-state_ini%lab)/real(tf-ts-1)
		do i = ts+1,tf-1
			state0(i)%lab = state0(i-1)%lab + inc
		end do
		!/* factor prices
		do i = ts+1,tf-1
			KoverH = state0(i)%cap/state0(i)%lab
			call factor_price(KoverH,tfp,state0(i)%theta,state0(i)%delta,state0(i)%rent,state0(i)%wage,KoverY)
		end do
		!/* payroll tax rate
		do i = ts+1,tf-1
			call govt_budget_ss(frac_ret(i),state0(i)%tax_ss)
		end do
		!/* labor income tax rate
		inc = (state_fin%tax_lab-state_ini%tax_lab)/real(tf-ts-1)
		do i = ts+1,tf-1
			state0(i)%tax_lab = state0(i-1)%tax_lab + inc
		end do
		!/* accidendal bequest
		inc = (state_fin%beq-state_ini%beq)/real(tf-ts-1)
		do i = ts+1,tf-1
			state0(i)%beq = state0(i-1)%beq + inc
		end do
		!/* government expenditure
		inc = (state_fin%gov_exp-state_ini%gov_exp)/real(tf-ts-1)
		do i = ts+1,tf-1
			state0(i)%gov_exp = state0(i-1)%gov_exp + inc
		end do
	end subroutine set_initial_path


	subroutine write_equilibrium_path
	!  Purpose:
	!  Output tentative results.
	!
	!  Record of revisions:
	!     Date      Programmer   Description of change
	!  ==========   ==========   =====================
	!  07/21/2008   T. Yamada    Original code
		implicit none
		!****** local variables ******
		integer :: i
		!-----------------------------
		open(21,file='eq_agg_capital.out')
		open(22,file='eq_agg_labor.out')
		open(23,file='eq_rent.out')
		open(24,file='eq_wage.out')
		open(25,file='eq_tax_ss.out')
		open(26,file='eq_tax_lab.out')
		open(27,file='eq_gov_exp.out')
		open(28,file='eq_acc_beq.out')
		open(31,file='exogenous_TFP.out')
		open(32,file='exogenous_hmax.out')
		open(33,file='exogenous_share.out')
		open(34,file='exogenous_depre.out')
		open(35,file='exogenous_tax_cap.out')
		open(36,file='exogenous_goy.out')
			do i = ts,tf
				write(21,*) state1(i)%cap
				write(22,*) state1(i)%lab
				write(23,*) state1(i)%rent
				write(24,*) state1(i)%wage
				write(25,*) state1(i)%tax_ss
				write(26,*) state1(i)%tax_lab
				write(27,*) state1(i)%gov_exp
				write(28,*) state1(i)%beq
				write(31,*) state1(i)%tfp_gr
				write(32,*) state1(i)%hmax
				write(33,*) state1(i)%theta
				write(34,*) state1(i)%delta
				write(35,*) state1(i)%tax_cap
				write(36,*) state1(i)%goy
			end do
		close(21)
		close(22)
		close(23)
		close(24)
		close(25)
		close(26)
		close(27)
		close(28)
		close(31)
		close(32)
		close(33)
		close(34)
		close(35)
		close(36)
	end subroutine write_equilibrium_path

end module mod_preamble