subroutine compute_inequality
!  Purpose:
!  Compute consumption and earnings inequality.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  03/31/2008   T. Yamada    Original code
!  11/10/2008   T. Yamada    Revised to submit JEDC
!  12/01/2010   T. Yamada    Reviesd to resubmit JEDC
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_preamble
	use mod_law_of_motion
 	implicit none
  
	!****** local variables ******
	integer :: i,j,k,l,it
	real(dp) :: lambda
	real(dp) :: K_supply,H_supply

	!****** density function ******
	type(Phi_function) :: phi,phi_new

	!****** age-capital profile ******
	real(dp),dimension(js:jf,ts:tf) :: K_age,K_mean_age
	real(dp),dimension(js:jf) :: K_next_age

	!****** age-labor profile ******
	real(dp),dimension(js:jr,ts:tf) :: H_age,H_mean_age
	real(dp),dimension(js:jr,ts:tf) :: H_hour_age

	!****** age-consumption profile ******
	real(dp),dimension(js:jf,ts:tf) :: C_age,C_mean_age

	!****** macro statistics ******
	real(dp),dimension(ts:tf) :: GDP,C_agg,G_agg,saving_rate

	!****** average hours worked ******
	real(dp),dimension(ts:tf) :: ave_hour_60,ave_hour_65

	!****** consumption, income and earning inequality ******
	real(dp),dimension(ts:tf) :: var_cons_60,var_erng_60,var_incm_60,var_work_60,var_cons_70,var_cons_75
	real(dp),dimension(ts:tf) :: var_cons_65,var_erng_65,var_incm_65,var_work_65
	real(dp),dimension(js:jf,ts:tf) :: var_cons_lc,var_erng_lc,var_incm_lc
	real(dp),dimension(js:jr,ts:tf) :: var_work_lc,corr_wh,frisch_age,labor_part

	!****** Frisch elasticity ******
	real(dp),dimension(ts:tf) :: frisch_65,frisch_60,corr_wh_65,corr_wh_60
	real(dp),dimension(ts:tf) :: ysl,csl
	!-------------------------------

	write(*,*) "-+-+-+- Computing economic inequality -+-+-+-"
	write(*,*) ""
	phi = phi_ini
	do it = ts,tf

		!/* asset distribution
		call law_of_motion_tr(it,policy(it),state0(it),phi,K_mean_age(:,it),K_next_age,H_mean_age(:,it),H_hour_age(:,it),phi_new)

		!/* update density
		phi = phi_new

		!/* consumption, income and earning inequality
		call cons_income_inequality(it,state0(it),phi,C_mean_age(:,it),&
		                          & var_cons_65(it),var_erng_65(it),var_incm_65(it),var_work_65(it),var_cons_70(it),var_cons_75(it),&
		                          & var_cons_60(it),var_erng_60(it),var_incm_60(it),var_work_60(it),&
		                          & var_cons_lc(:,it),var_erng_lc(:,it),var_incm_lc(:,it),var_work_lc(:,it),&
		                          & ysl(it),csl(it),labor_part(:,it))

		!/* correlation btwn wage and hours worked
		call corr_wage_hours(it,state0(it),phi,corr_wh_65(it),corr_wh_60(it),corr_wh(:,it))

		!/* aggregate capital, labor and consumption
		do i = js,jf
			K_age(i,it) = mu(i,it)*K_mean_age(i,it)
			C_age(i,it) = mu(i,it)*C_mean_age(i,it)
		end do
		do i = js,jr
			H_age(i,it) = mu(i,it)*H_mean_age(i,it)
		end do
		K_supply  = sum(K_age(:,it))
		H_supply  = sum(H_age(:,it))
		C_agg(it) = sum(C_age(:,it))
		G_agg(it) = state0(it)%goy*(tfp*(K_supply**state0(it)%theta)*(H_supply**(1.0-state0(it)%theta)))

		!/* average hours worked: 20-60 & 20-65
		ave_hour_60(it) = 0.0
		do i = js,60
			ave_hour_60(it) = (mu(i,it)/sum(mu(js:60,it),dim=1))*H_hour_age(i,it) + ave_hour_60(it)
		end do
		ave_hour_65(it) = 0.0
		do i = js,65
			ave_hour_65(it) = (mu(i,it)/sum(mu(js:65,it),dim=1))*H_hour_age(i,it) + ave_hour_65(it)
		end do

		!/* output and saving rate
		GDP(it) = tfp*(K_supply**state0(it)%theta)*(H_supply**(1.0-state0(it)%theta))
		saving_rate(it) = (GDP(it)-C_agg(it)-G_agg(it)-state0(it)%delta*K_supply) / (GDP(it)-state0(it)%delta*K_supply)

		!/* Frisch elasticity
		lambda = (1.0-sigma+sigma*gamma)/gamma
		frisch_65(it) = lambda*((state0(it)%hmax-ave_hour_65(it))/ave_hour_65(it))
		frisch_60(it) = lambda*((state0(it)%hmax-ave_hour_60(it))/ave_hour_60(it))
		do i = js,jr
			frisch_age(i,it) = lambda*((state0(it)%hmax-H_hour_age(i,it))/H_hour_age(i,it))
		end do

	end do

	!/* output results */
	open(24,file='eq_output.out')
	open(25,file='eq_consumption.out')
	open(26,file='eq_saving_rate.out')
	open(27,file='eq_hour_60.out')
	open(28,file='eq_hour_65.out')
	open(31,file='var_consumption_65.out')
	open(32,file='var_earning_65.out')
	open(33,file='var_income_65.out')
	open(34,file='var_hours_65.out')
	open(35,file='var_consumption_70.out')
	open(36,file='var_consumption_75.out')
	open(41,file='var_consumption_60.out')
	open(42,file='var_earning_60.out')
	open(43,file='var_income_60.out')
	open(44,file='var_hours_60.out')
	open(51,file='path_slope_earning.out')
	open(52,file='path_slope_consumption.out')
	open(61,file='path_corr_wh_65.out')
	open(62,file='path_corr_wh_60.out')
	open(63,file='path_frisch_65.out')
	open(64,file='path_frisch_60.out')
!	open(71,file='age_corr_wh.out')
!	open(72,file='age_frisch.out')
!	open(73,file='age_labor_participation.out')
		do it = ts,tf
			write(24,*) GDP(it)
			write(25,*) C_agg(it)
			write(26,*) saving_rate(it)
			write(27,*) ave_hour_60(it)
			write(28,*) ave_hour_65(it)
			write(31,*) var_cons_65(it)
			write(32,*) var_erng_65(it)
			write(33,*) var_incm_65(it)
			write(34,*) var_work_65(it)
			write(35,*) var_cons_70(it)
			write(36,*) var_cons_75(it)
			write(41,*) var_cons_60(it)
			write(42,*) var_erng_60(it)
			write(43,*) var_incm_60(it)
			write(44,*) var_work_60(it)
			write(51,*) ysl(it)
			write(52,*) csl(it)
			write(61,*) corr_wh_65(it)
			write(62,*) corr_wh_60(it)
			write(63,*) frisch_65(it)
			write(64,*) frisch_60(it)
!			do i = js,jr
!				write(71,*) corr_wh(i,it)
!				write(72,*) frisch_age(i,it)
!			end do
!			do i = js,jr
!				write(73,*) labor_part(i,it)
!			end do
		end do
	close(24)
	close(25)
	close(26)
	close(27)
	close(28)
	close(31)
	close(32)
	close(33)
	close(34)
	close(35)
	close(36)
	close(41)
	close(42)
	close(43)
	close(44)
	close(51)
	close(52)
	close(61)
	close(62)
	close(63)
	close(64)
!	close(71)
!	close(72)
!	close(73)

	!/* cohort history: consumption
	open(11,file='cohort_c_1980_20.txt')
	open(12,file='cohort_c_1980_25.txt')
	open(13,file='cohort_c_1980_30.txt')
	open(14,file='cohort_c_1980_35.txt')
	open(15,file='cohort_c_1980_40.txt')
	open(16,file='cohort_c_1980_45.txt')
	open(17,file='cohort_c_1980_50.txt')
	open(18,file='cohort_c_1980_55.txt')
	open(19,file='cohort_c_1980_60.txt')
	open(20,file='cohort_c_1980_65.txt')
	open(21,file='cohort_c_1985_20.txt')
	open(22,file='cohort_c_1990_20.txt')
	open(23,file='cohort_c_1995_20.txt')
		do it = 20,70
			write(11,*) C_mean_age(it,it+1960)
		end do
		do it = 25,70
			write(12,*) C_mean_age(it,it+1955)
		end do
		do it = 30,70
			write(13,*) C_mean_age(it,it+1950)
		end do
		do it = 35,70
			write(14,*) C_mean_age(it,it+1945)
		end do
		do it = 40,70
			write(15,*) C_mean_age(it,it+1940)
		end do
		do it = 45,70
			write(16,*) C_mean_age(it,it+1935)
		end do
		do it = 50,70
			write(17,*) C_mean_age(it,it+1930)
		end do
		do it = 55,70
			write(18,*) C_mean_age(it,it+1925)
		end do
		do it = 60,70
			write(19,*) C_mean_age(it,it+1920)
		end do
		do it = 65,70
			write(20,*) C_mean_age(it,it+1915)
		end do
		do it = 20,70
			write(21,*) C_mean_age(it,it+1965)
		end do
		do it = 20,70
			write(22,*) C_mean_age(it,it+1970)
		end do
		do it = 20,70
			write(23,*) C_mean_age(it,it+1975)
		end do
	close(11)
	close(12)
	close(13)
	close(14)
	close(15)
	close(16)
	close(17)
	close(18)
	close(19)
	close(20)
	close(21)
	close(22)
	close(23)

	!/* cohort history: earnings
	open(11,file='cohort_y_1980_20.txt')
	open(12,file='cohort_y_1980_25.txt')
	open(13,file='cohort_y_1980_30.txt')
	open(14,file='cohort_y_1980_35.txt')
	open(15,file='cohort_y_1980_40.txt')
	open(16,file='cohort_y_1980_45.txt')
	open(17,file='cohort_y_1980_50.txt')
	open(18,file='cohort_y_1980_55.txt')
	open(19,file='cohort_y_1980_60.txt')
	open(20,file='cohort_y_1985_20.txt')
	open(21,file='cohort_y_1990_20.txt')
	open(22,file='cohort_y_1995_20.txt')
		do it = 20,65
			write(11,*) H_mean_age(it,it+1960)
		end do
		do it = 25,65
			write(12,*) H_mean_age(it,it+1955)
		end do
		do it = 30,65
			write(13,*) H_mean_age(it,it+1950)
		end do
		do it = 35,65
			write(14,*) H_mean_age(it,it+1945)
		end do
		do it = 40,65
			write(15,*) H_mean_age(it,it+1940)
		end do
		do it = 45,65
			write(16,*) H_mean_age(it,it+1935)
		end do
		do it = 50,65
			write(17,*) H_mean_age(it,it+1930)
		end do
		do it = 55,65
			write(18,*) H_mean_age(it,it+1925)
		end do
		do it = 60,65
			write(19,*) H_mean_age(it,it+1920)
		end do
		do it = 20,65
			write(20,*) H_mean_age(it,it+1965)
		end do
		do it = 20,65
			write(21,*) H_mean_age(it,it+1970)
		end do
		do it = 20,65
			write(22,*) H_mean_age(it,it+1975)
		end do
	close(11)
	close(12)
	close(13)
	close(14)
	close(15)
	close(16)
	close(17)
	close(18)
	close(19)
	close(20)
	close(21)
	close(22)

	!/* cohort history: hours worked
	open(11,file='cohort_h_1980_20.txt')
	open(12,file='cohort_h_1980_25.txt')
	open(13,file='cohort_h_1980_30.txt')
	open(14,file='cohort_h_1980_35.txt')
	open(15,file='cohort_h_1980_40.txt')
	open(16,file='cohort_h_1980_45.txt')
	open(17,file='cohort_h_1980_50.txt')
	open(18,file='cohort_h_1980_55.txt')
	open(19,file='cohort_h_1980_60.txt')
	open(20,file='cohort_h_1985_20.txt')
	open(21,file='cohort_h_1990_20.txt')
	open(22,file='cohort_h_1995_20.txt')
		do it = 20,80
			write(11,*) H_hour_age(it,it+1960)
		end do
		do it = 25,65
			write(12,*) H_hour_age(it,it+1955)
		end do
		do it = 30,65
			write(13,*) H_hour_age(it,it+1950)
		end do
		do it = 35,65
			write(14,*) H_hour_age(it,it+1945)
		end do
		do it = 40,65
			write(15,*) H_hour_age(it,it+1940)
		end do
		do it = 45,65
			write(16,*) H_hour_age(it,it+1935)
		end do
		do it = 50,65
			write(17,*) H_hour_age(it,it+1930)
		end do
		do it = 55,65
			write(18,*) H_hour_age(it,it+1925)
		end do
		do it = 60,65
			write(19,*) H_hour_age(it,it+1920)
		end do
		do it = 20,65
			write(20,*) H_hour_age(it,it+1965)
		end do
		do it = 20,65
			write(21,*) H_hour_age(it,it+1970)
		end do
		do it = 20,65
			write(22,*) H_hour_age(it,it+1975)
		end do
	close(11)
	close(12)
	close(13)
	close(14)
	close(15)
	close(16)
	close(17)
	close(18)
	close(19)
	close(20)
	close(21)
	close(22)

	!/* cohort history: var. of log-earnings
	open(11,file='cohort_vy_1980_20.txt')
	open(12,file='cohort_vy_1980_25.txt')
	open(13,file='cohort_vy_1980_30.txt')
	open(14,file='cohort_vy_1980_35.txt')
	open(15,file='cohort_vy_1980_40.txt')
	open(16,file='cohort_vy_1980_45.txt')
	open(17,file='cohort_vy_1980_50.txt')
	open(18,file='cohort_vy_1980_55.txt')
	open(19,file='cohort_vy_1980_60.txt')
	open(20,file='cohort_vy_1980_65.txt')
	open(21,file='cohort_vy_1985_20.txt')
	open(22,file='cohort_vy_1990_20.txt')
	open(23,file='cohort_vy_1995_20.txt')
		do it = 20,70
			write(11,*) var_erng_lc(it,it+1960)
		end do
		do it = 25,70
			write(12,*) var_erng_lc(it,it+1955)
		end do
		do it = 30,70
			write(13,*) var_erng_lc(it,it+1950)
		end do
		do it = 35,70
			write(14,*) var_erng_lc(it,it+1945)
		end do
		do it = 40,70
			write(15,*) var_erng_lc(it,it+1940)
		end do
		do it = 45,70
			write(16,*) var_erng_lc(it,it+1935)
		end do
		do it = 50,70
			write(17,*) var_erng_lc(it,it+1930)
		end do
		do it = 55,70
			write(18,*) var_erng_lc(it,it+1925)
		end do
		do it = 60,70
			write(19,*) var_erng_lc(it,it+1920)
		end do
		do it = 65,70
			write(20,*) var_erng_lc(it,it+1915)
		end do
		do it = 20,70
			write(21,*) var_erng_lc(it,it+1965)
		end do
		do it = 20,70
			write(22,*) var_erng_lc(it,it+1970)
		end do
		do it = 20,70
			write(23,*) var_erng_lc(it,it+1975)
		end do
	close(11)
	close(12)
	close(13)
	close(14)
	close(15)
	close(16)
	close(17)
	close(18)
	close(19)
	close(20)
	close(21)
	close(22)
	close(23)

	!/* cohort history: var. of log-consumption
	open(11,file='cohort_vc_1980_20.txt')
	open(12,file='cohort_vc_1980_25.txt')
	open(13,file='cohort_vc_1980_30.txt')
	open(14,file='cohort_vc_1980_35.txt')
	open(15,file='cohort_vc_1980_40.txt')
	open(16,file='cohort_vc_1980_45.txt')
	open(17,file='cohort_vc_1980_50.txt')
	open(18,file='cohort_vc_1980_55.txt')
	open(19,file='cohort_vc_1980_60.txt')
	open(20,file='cohort_vc_1980_65.txt')
	open(21,file='cohort_vc_1985_20.txt')
	open(22,file='cohort_vc_1990_20.txt')
	open(23,file='cohort_vc_1995_20.txt')
		do it = 20,70
			write(11,*) var_cons_lc(it,it+1960)
		end do
		do it = 25,70
			write(12,*) var_cons_lc(it,it+1955)
		end do
		do it = 30,70
			write(13,*) var_cons_lc(it,it+1950)
		end do
		do it = 35,70
			write(14,*) var_cons_lc(it,it+1945)
		end do
		do it = 40,70
			write(15,*) var_cons_lc(it,it+1940)
		end do
		do it = 45,70
			write(16,*) var_cons_lc(it,it+1935)
		end do
		do it = 50,70
			write(17,*) var_cons_lc(it,it+1930)
		end do
		do it = 55,70
			write(18,*) var_cons_lc(it,it+1925)
		end do
		do it = 60,70
			write(19,*) var_cons_lc(it,it+1920)
		end do
		do it = 65,70
			write(20,*) var_cons_lc(it,it+1915)
		end do
		do it = 20,70
			write(21,*) var_cons_lc(it,it+1965)
		end do
		do it = 20,70
			write(22,*) var_cons_lc(it,it+1970)
		end do
		do it = 20,70
			write(23,*) var_cons_lc(it,it+1975)
		end do
	close(11)
	close(12)
	close(13)
	close(14)
	close(15)
	close(16)
	close(17)
	close(18)
	close(19)
	close(20)
	close(21)
	close(22)
	close(23)

end subroutine compute_inequality


subroutine cons_income_inequality(time,state,phi,cons_mean_path,&
                                  & var_cons_65,var_erng_65,var_incm_65,var_work_65,var_cons_70,var_cons_75,&
                                  & var_cons_60,var_erng_60,var_incm_60,var_work_60,&
                                  & var_cons_lc,var_erng_lc,var_incm_lc,var_work_lc,&
                                  & yslope,cslope,labor_part)
!  Purpose:
!  Variance of consumption, earning and income.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  11/05/2006   T. Yamada    Original code
!  04/05/2008   T. Yamada    Revised to submit JEDC
!  10/24/2010   T. Yamada    Reviesd to resubmit JEDC
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_law_of_motion
 	implicit none

	!****** input ******
	integer,intent(in) :: time
	type(state_space),intent(in)  :: state
	type(phi_function),intent(in) :: phi

	!****** output ******
	real(dp),intent(out),dimension(js:jf) :: cons_mean_path
	real(dp),intent(out) :: var_cons_65,var_erng_65,var_incm_65,var_work_65,var_cons_70,var_cons_75
	real(dp),intent(out) :: var_cons_60,var_erng_60,var_incm_60,var_work_60
	real(dp),intent(out),dimension(js:jf) :: var_cons_lc,var_erng_lc,var_incm_lc
	real(dp),intent(out),dimension(js:jr) :: var_work_lc,labor_part
	real(dp),intent(out) :: yslope,cslope

	!****** local variables ******
	integer :: i,j,k,l,ia,status
	real(dp) :: wealth,wage0,rent0,tax_ss0,tax_lab0,H_agg0,beq0
	real(dp) :: tfp_gr0,hmax0,tax_cap0
	real(dp) :: sqr_erng_60,sqr_incm_60,sqr_work_60
	real(dp) :: avg_erng_60,avg_incm_60,avg_work_60
	real(dp) :: sqr_erng_65,sqr_incm_65,sqr_work_65
	real(dp) :: avg_erng_65,avg_incm_65,avg_work_65
	real(dp) :: avg_cons_60,avg_cons_65,avg_cons_70,avg_cons_75
	real(dp) :: sqr_cons_60,sqr_cons_65,sqr_cons_70,sqr_cons_75
	real(dp),dimension(js:jf) :: avg_cons,avg_erng,avg_incm,avg_work
	real(dp),dimension(js:jf) :: sqr_cons,sqr_erng,sqr_incm,sqr_work
	real(dp),dimension(js:jf) :: zlc,zly,zli,zlh,erng_mean_path
	real(dp),allocatable,dimension(:,:,:,:,:) :: log_cons1,log_erng1,log_incm1,log_work1
	real(dp),allocatable,dimension(:,:,:,:,:) :: saving1,cons1,earning1,income1,hours
	real(dp),allocatable,dimension(:,:,:) :: log_cons2,log_erng2,log_incm2
	real(dp),allocatable,dimension(:,:,:) :: saving2,cons2,earning2,income2
	!-----------------------------

	tfp_gr0  = state%tfp_gr
	hmax0    = state%hmax
	tax_cap0 = state%tax_cap
	rent0    = state%rent
	wage0    = state%wage
	tax_ss0  = state%tax_ss
	tax_lab0 = state%tax_lab
	H_agg0   = state%lab
	beq0     = state%beq

	allocate(log_cons1(nd,nz,nt,nf,js:jf),STAT=status)
	allocate(log_erng1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_incm1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_work1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_cons2(nd,nf,jr+1:jf),STAT=status)
	allocate(log_erng2(nd,nf,jr+1:jf),STAT=status)
	allocate(log_incm2(nd,nf,jr+1:jf),STAT=status)
	log_cons1 = 0.0
	log_cons2 = 0.0
	log_erng1 = 0.0
	log_erng2 = 0.0
	log_incm1 = 0.0
	log_incm2 = 0.0
	log_work1 = 0.0

	allocate(saving1(nd,nz,nt,nf,js:jf),STAT=status)
	allocate(saving2(nd,nf,jr+1:jf),STAT=status)
	allocate(cons1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(cons2(nd,nf,jr+1:jf),STAT=status)
	allocate(earning1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(earning2(nd,nf,jr+1:jf),STAT=status)
	allocate(income1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(income2(nd,nf,jr+1:jf),STAT=status)
	allocate(hours(nd,nz,nt,nf,js:jr),STAT=status)
	saving1  = 0.0
	saving2  = 0.0
	cons1    = 0.0
	cons2    = 0.0
	earning1 = 0.0
	earning2 = 0.0
	income1  = 0.0
	income2  = 0.0
	hours = 0.0

	!/* re-compute policy function for finer grids
	call finer_policy(policy(time),hmax0,saving1,hours,saving2)

	!/* consumption, earnings & disposable income
	labor_part = 0.0
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							earning1(i,j,k,l,ia) = wage0*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hours(i,j,k,l,ia)
							income1(i,j,k,l,ia)  = (1.0-tax_cap0)*rent0*(grcap2(i)+beq0) + (1.0-tax_ss0-tax_lab0)*earning1(i,j,k,l,ia)
							wealth               = (1.0+(1.0-tax_cap0)*rent0)*(grcap2(i)+beq0) + (1.0-tax_ss0-tax_lab0)*earning1(i,j,k,l,ia)
							cons1(i,j,k,l,ia)    = wealth - tfp_gr0*saving1(i,j,k,l,ia)
							if (hours(i,j,k,l,ia) > 0.0) then
								labor_part(ia) = phi%y(i,j,k,l,ia) + labor_part(ia)
							end if
						end do
					end do
				end do
			end do
		case(jr+1:jf)
			do l = 1,nf
				do i = 1,nd
					earning2(i,l,ia) = rep_rate*wage0*H_agg0*shock_fx(l)
					income2(i,l,ia)  = (1.0-tax_cap0)*rent0*(grcap2(i)+beq0) + earning2(i,l,ia)
					wealth           = (1.0+(1.0-tax_cap0)*rent0)*(grcap2(i)+beq0) + earning2(i,l,ia)
					cons2(i,l,ia)    = wealth - tfp_gr0*saving2(i,l,ia)
				end do
			end do
		end select
	end do

	!/* average consumption profiles
	cons_mean_path = 0.0
	do l = 1,nf
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							cons_mean_path(ia) = phi%y(i,j,k,l,ia)*cons1(i,j,k,l,ia) + cons_mean_path(ia)
							erng_mean_path(ia) = phi%y(i,j,k,l,ia)*earning1(i,j,k,l,ia) + erng_mean_path(ia)
						end do
					end do
				end do
			case(jr+1:jf)
				do i = 1,nd
					cons_mean_path(ia) = phi%o(i,l,ia)*cons2(i,l,ia) + cons_mean_path(ia)
					erng_mean_path(ia) = phi%o(i,l,ia)*earning2(i,l,ia) + erng_mean_path(ia)
				end do
			end select
		end do
	end do

	!/* cross section of log-consumption and income profile
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							if (cons1(i,j,k,l,ia) > 0.0) then
								log_cons1(i,j,k,l,ia) = log(cons1(i,j,k,l,ia))
							else
								log_cons1(i,j,k,l,ia) = 0.0
								zlc(ia) = phi%y(i,j,k,l,ia) + zlc(ia)
							end if
							if (earning1(i,j,k,l,ia) > 0.0) then
								log_erng1(i,j,k,l,ia) = log(earning1(i,j,k,l,ia))
							else
								log_erng1(i,j,k,l,ia) = 0.0
								zly(ia) = phi%y(i,j,k,l,ia) + zly(ia)
							end if
							if (income1(i,j,k,l,ia) > 0.0) then
								log_incm1(i,j,k,l,ia) = log(income1(i,j,k,l,ia))
							else
								log_incm1(i,j,k,l,ia) = 0.0
								zli(ia) = phi%y(i,j,k,l,ia) + zli(ia)
							end if
							if (hours(i,j,k,l,ia) > 0.0) then
								log_work1(i,j,k,l,ia) = log(hours(i,j,k,l,ia))
							else
								log_work1(i,j,k,l,ia) = 0.0
								zlh(ia) = phi%y(i,j,k,l,ia) + zlh(ia)
							end if
						end do
					end do
				end do
			end do
		case(jr+1:jf)
			do l = 1,nf
				do i = 1,nd
					if (cons2(i,l,ia) > 0) then
						log_cons2(i,l,ia) = log(cons2(i,l,ia))
					else
						log_cons2(i,l,ia) = 0.0
						zlc(ia) = phi%y(i,j,k,l,ia) + zlc(ia)
					end if
					if (earning2(i,l,ia) > 0) then
						log_erng2(i,l,ia) = log(earning2(i,l,ia))
					else
						log_erng2(i,l,ia) = 0.0
						zly(ia) = phi%y(i,j,k,l,ia) + zly(ia)
					end if
					if (income2(i,l,ia) > 0) then
						log_incm2(i,l,ia) = log(income2(i,l,ia))
					else
						log_incm2(i,l,ia) = 0.0
						zli(ia) = phi%y(i,j,k,l,ia) + zli(ia)
					end if
				end do
			end do
		end select
	end do

	!/* variances of log of consumption etc. by age
	sqr_cons = 0.0
	sqr_erng = 0.0
	sqr_incm = 0.0
	sqr_work = 0.0
	avg_cons = 0.0
	avg_erng = 0.0
	avg_incm = 0.0
	avg_work = 0.0
	do l = 1,nf
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							sqr_cons(ia) = phi%y(i,j,k,l,ia)*(log_cons1(i,j,k,l,ia)**2) + sqr_cons(ia)
							sqr_erng(ia) = phi%y(i,j,k,l,ia)*(log_erng1(i,j,k,l,ia)**2) + sqr_erng(ia)
							sqr_incm(ia) = phi%y(i,j,k,l,ia)*(log_incm1(i,j,k,l,ia)**2) + sqr_incm(ia)
							sqr_work(ia) = phi%y(i,j,k,l,ia)*(log_work1(i,j,k,l,ia)**2) + sqr_work(ia)
							avg_cons(ia) = phi%y(i,j,k,l,ia)*log_cons1(i,j,k,l,ia) + avg_cons(ia)
							avg_erng(ia) = phi%y(i,j,k,l,ia)*log_erng1(i,j,k,l,ia) + avg_erng(ia)
							avg_incm(ia) = phi%y(i,j,k,l,ia)*log_incm1(i,j,k,l,ia) + avg_incm(ia)
							avg_work(ia) = phi%y(i,j,k,l,ia)*log_work1(i,j,k,l,ia) + avg_work(ia)
						end do
					end do
				end do
			case(jr+1:jf)
				do i = 1,nd
					sqr_cons(ia) = phi%o(i,l,ia)*(log_cons2(i,l,ia)**2) + sqr_cons(ia)
					sqr_erng(ia) = phi%o(i,l,ia)*(log_erng2(i,l,ia)**2) + sqr_erng(ia)
					sqr_incm(ia) = phi%o(i,l,ia)*(log_incm2(i,l,ia)**2) + sqr_incm(ia)
					avg_cons(ia) = phi%o(i,l,ia)*log_cons2(i,l,ia) + avg_cons(ia)
					avg_erng(ia) = phi%o(i,l,ia)*log_erng2(i,l,ia) + avg_erng(ia)
					avg_incm(ia) = phi%o(i,l,ia)*log_incm2(i,l,ia) + avg_incm(ia)
				end do
			end select
		end do
	end do
	var_cons_lc = 0.0
	var_erng_lc = 0.0
	var_incm_lc = 0.0
	var_work_lc = 0.0
	do ia = js,jf
		var_cons_lc(ia) = sqr_cons(ia) - avg_cons(ia)**2
		var_erng_lc(ia) = sqr_erng(ia) - avg_erng(ia)**2
		var_incm_lc(ia) = sqr_incm(ia) - avg_incm(ia)**2
	end do
	do ia = js,jr
		var_work_lc(ia) = sqr_work(ia) - avg_work(ia)**2
	end do

	!/* economy-wide inequality
	sqr_cons_60 = 0.0
	sqr_erng_60 = 0.0
	sqr_incm_60 = 0.0
	sqr_work_60 = 0.0
	avg_cons_60 = 0.0
	avg_erng_60 = 0.0
	avg_incm_60 = 0.0
	avg_work_60 = 0.0
	sqr_cons_65 = 0.0
	sqr_erng_65 = 0.0
	sqr_incm_65 = 0.0
	sqr_work_65 = 0.0
	avg_cons_65 = 0.0
	avg_erng_65 = 0.0
	avg_incm_65 = 0.0
	avg_work_65 = 0.0
	avg_cons_70 = 0.0
	sqr_cons_70 = 0.0
	avg_cons_75 = 0.0
	sqr_cons_75 = 0.0
	do ia = js,60
		sqr_cons_60 = mu(ia,time)/sum(mu(js:60,time),dim=1)*sqr_cons(ia) + sqr_cons_60
		sqr_erng_60 = mu(ia,time)/sum(mu(js:60,time),dim=1)*sqr_erng(ia) + sqr_erng_60
		sqr_incm_60 = mu(ia,time)/sum(mu(js:60,time),dim=1)*sqr_incm(ia) + sqr_incm_60
		sqr_work_60 = mu(ia,time)/sum(mu(js:60,time),dim=1)*sqr_work(ia) + sqr_work_60
		avg_cons_60 = mu(ia,time)/sum(mu(js:60,time),dim=1)*avg_cons(ia) + avg_cons_60
		avg_erng_60 = mu(ia,time)/sum(mu(js:60,time),dim=1)*avg_erng(ia) + avg_erng_60
		avg_incm_60 = mu(ia,time)/sum(mu(js:60,time),dim=1)*avg_incm(ia) + avg_incm_60
		avg_work_60 = mu(ia,time)/sum(mu(js:60,time),dim=1)*avg_work(ia) + avg_work_60
	end do
	do ia = js,65
		sqr_cons_65 = mu(ia,time)/sum(mu(js:65,time),dim=1)*sqr_cons(ia) + sqr_cons_65
		sqr_erng_65 = mu(ia,time)/sum(mu(js:65,time),dim=1)*sqr_erng(ia) + sqr_erng_65
		sqr_incm_65 = mu(ia,time)/sum(mu(js:65,time),dim=1)*sqr_incm(ia) + sqr_incm_65
		sqr_work_65 = mu(ia,time)/sum(mu(js:65,time),dim=1)*sqr_work(ia) + sqr_work_65
		avg_cons_65 = mu(ia,time)/sum(mu(js:65,time),dim=1)*avg_cons(ia) + avg_cons_65
		avg_erng_65 = mu(ia,time)/sum(mu(js:65,time),dim=1)*avg_erng(ia) + avg_erng_65
		avg_incm_65 = mu(ia,time)/sum(mu(js:65,time),dim=1)*avg_incm(ia) + avg_incm_65
		avg_work_65 = mu(ia,time)/sum(mu(js:65,time),dim=1)*avg_work(ia) + avg_work_65
	end do
	do ia = js,70
		sqr_cons_70 = mu(ia,time)/sum(mu(js:70,time),dim=1)*sqr_cons(ia) + sqr_cons_70
		avg_cons_70 = mu(ia,time)/sum(mu(js:70,time),dim=1)*avg_cons(ia) + avg_cons_70
	end do
	do ia = js,75
		sqr_cons_75 = mu(ia,time)/sum(mu(js:75,time),dim=1)*sqr_cons(ia) + sqr_cons_75
		avg_cons_75 = mu(ia,time)/sum(mu(js:75,time),dim=1)*avg_cons(ia) + avg_cons_75
	end do
	var_cons_60 = sqr_cons_60 - avg_cons_60**2
	var_cons_65 = sqr_cons_65 - avg_cons_65**2
	var_cons_70 = sqr_cons_70 - avg_cons_70**2
	var_cons_75 = sqr_cons_75 - avg_cons_75**2
	var_incm_60 = sqr_incm_60 - avg_incm_60**2
	var_incm_65 = sqr_incm_65 - avg_incm_65**2
	var_erng_60 = sqr_erng_60 - avg_erng_60**2
	var_erng_65 = sqr_erng_65 - avg_erng_65**2
	var_work_60 = sqr_work_60 - avg_work_60**2
	var_work_65 = sqr_work_65 - avg_work_65**2

	!/* slopes of earnings and consumption
	yslope = (sum(erng_mean_path(45:55))/11.0) / (sum(erng_mean_path(25:35))/11.0)
	cslope = (sum(cons_mean_path(45:55))/11.0) / (sum(cons_mean_path(25:35))/11.0)

	deallocate(log_cons1)
	deallocate(log_cons2)
	deallocate(log_erng1)
	deallocate(log_erng2)
	deallocate(log_incm1)
	deallocate(log_incm2)
	deallocate(log_work1)
	deallocate(saving1)
	deallocate(saving2)
	deallocate(cons1)
	deallocate(cons2)
	deallocate(earning1)
	deallocate(earning2)
	deallocate(income1)
	deallocate(income2)
	deallocate(hours)

end subroutine cons_income_inequality


subroutine corr_wage_hours(time,state,phi,corr_wh_65,corr_wh_60,corr_wh)
!  Purpose:
!  Correlation between wage and hours.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  10/24/2010   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_law_of_motion
	use mod_interpolation
 	implicit none
 
	!****** input ******
	integer,intent(in) :: time
	type(state_space),intent(in) :: state
	type(phi_function),intent(in) :: phi

	!****** output ******
	real(dp),intent(out) :: corr_wh_65,corr_wh_60
	real(dp),dimension(js:jr),intent(out) :: corr_wh

	!****** local variables ******
	integer :: i,j,k,l,ia,status
	real(dp) :: ww,wage0,hmax0
	real(dp) :: avg_work_65,var_work_65,avg_wage_65,var_wage_65,cov_wh_65
	real(dp) :: avg_work_60,var_work_60,avg_wage_60,var_wage_60,cov_wh_60
	real(dp),dimension(js:jf) :: avg_work,var_work,avg_wage,var_wage,cov_wh
	real(dp),allocatable,dimension(:,:,:,:,:) :: log_work1,log_wage1
	real(dp),allocatable,dimension(:,:,:,:,:) :: saving1,hours
	real(dp),allocatable,dimension(:,:,:) :: saving2
	!-----------------------------

	hmax0 = state%hmax
	wage0 = state%wage

	allocate(log_work1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_wage1(nd,nz,nt,nf,js:jr),STAT=status)
	log_work1 = 0.0
	log_wage1 = 0.0
	allocate(saving1(nd,nz,nt,nf,js:jf),STAT=status)
	allocate(saving2(nd,nf,jr+1:jf),STAT=status)
	allocate(hours(nd,nz,nt,nf,js:jr),STAT=status)
	saving1 = 0.0
	saving2 = 0.0
	hours = 0.0

	!/* re-compute policy function over finer grids
	call finer_policy(policy(time),hmax0,saving1,hours,saving2)
	deallocate(saving1)
	deallocate(saving2)
	
	!/* log-wage and log-hours
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						ww = wage0*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)
						if (hours(i,j,k,l,ia) > 0.0) then
							log_wage1(i,j,k,l,ia) = log(ww)
							log_work1(i,j,k,l,ia) = log(hours(i,j,k,l,ia))
						else
							log_wage1(i,j,k,l,ia) = 0.0
							log_work1(i,j,k,l,ia) = 0.0
						end if
					end do
				end do
			end do
		end do
	end do

	!/* mean of log-wage and log-hours
	avg_wage = 0.0
	avg_work = 0.0
	avg_wage_65 = 0.0
	avg_work_65 = 0.0
	avg_wage_60 = 0.0
	avg_work_60 = 0.0
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						avg_wage(ia) = phi%y(i,j,k,l,ia)*log_wage1(i,j,k,l,ia) + avg_wage(ia)
						avg_work(ia) = phi%y(i,j,k,l,ia)*log_work1(i,j,k,l,ia) + avg_work(ia)
						avg_wage_65 = (mu(ia,time)/sum(mu(js:65,time),dim=1))*phi%y(i,j,k,l,ia)*log_wage1(i,j,k,l,ia) + avg_wage_65
						avg_work_65 = (mu(ia,time)/sum(mu(js:65,time),dim=1))*phi%y(i,j,k,l,ia)*log_work1(i,j,k,l,ia) + avg_work_65
						if (ia <= 60) then
							avg_wage_60 = (mu(ia,time)/sum(mu(js:60,time),dim=1))*phi%y(i,j,k,l,ia)*log_wage1(i,j,k,l,ia) + avg_wage_60
							avg_work_60 = (mu(ia,time)/sum(mu(js:60,time),dim=1))*phi%y(i,j,k,l,ia)*log_work1(i,j,k,l,ia) + avg_work_60
						end if
					end do
				end do
			end do
		end do
	end do

	!/* correlation btw log-wage and log-hours */
	var_wage = 0.0
	var_work = 0.0
	cov_wh   = 0.0
	var_wage_65 = 0.0
	var_work_65 = 0.0
	cov_wh_65 = 0.0
	var_wage_60 = 0.0
	var_work_60 = 0.0
	cov_wh_60 = 0.0
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						var_wage(ia) = phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage(ia))**2) + var_wage(ia)
						var_work(ia) = phi%y(i,j,k,l,ia)*((log_work1(i,j,k,l,ia)-avg_work(ia))**2) + var_work(ia)
						cov_wh(ia)   = phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage(ia))*(log_work1(i,j,k,l,ia)-var_work(ia))) + cov_wh(ia)
						var_wage_65 = (mu(ia,time)/sum(mu(js:65,time),dim=1))*phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage_65)**2) + var_wage_65
						var_work_65 = (mu(ia,time)/sum(mu(js:65,time),dim=1))*phi%y(i,j,k,l,ia)*((log_work1(i,j,k,l,ia)-avg_work_65)**2) + var_work_65
						cov_wh_65   = (mu(ia,time)/sum(mu(js:65,time),dim=1))*phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage_65)*(log_work1(i,j,k,l,ia)-avg_work_65)) + cov_wh_65
						if (ia <= 60) then
							var_wage_60 = (mu(ia,time)/sum(mu(js:60,time),dim=1))*phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage_60)**2) + var_wage_60
							var_work_60 = (mu(ia,time)/sum(mu(js:60,time),dim=1))*phi%y(i,j,k,l,ia)*((log_work1(i,j,k,l,ia)-avg_work_60)**2) + var_work_60
							cov_wh_60   = (mu(ia,time)/sum(mu(js:60,time),dim=1))*phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage_60)*(log_work1(i,j,k,l,ia)-avg_work_60)) + cov_wh_60
						end if
					end do
				end do
			end do
		end do
	end do

	!/* corr(w,h): age-dependent
	do ia = js,jr
		corr_wh(ia) = sqrt((cov_wh(ia)**2)/(var_wage(ia)*var_work(ia)))
	end do
	corr_wh_65 = sqrt((cov_wh_65**2)/(var_wage_65*var_work_65))
	corr_wh_60 = sqrt((cov_wh_60**2)/(var_wage_60*var_work_60))

	deallocate(log_work1)
	deallocate(log_wage1)
	deallocate(hours)

end subroutine corr_wage_hours