subroutine steady_state_ini
!  Purpose:
!  Compute an initial steady state of a lifecyle model.
!
!  Record of Revisions:
!     Date      Programmer   Description of Change
!  ==========   ==========   =====================
!  02/02/2011   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_preamble
	use mod_end_grid_method
	use mod_law_of_motion
	implicit none

	!****** main loop ******
	integer :: it_gy,it_kl,i,j ,k,l,ia
	real(dp) :: metric_cap = 1.0
	real(dp) :: metric_lab = 1.0
	real(dp) :: metric_gy = 1.0
	integer,parameter :: maxit = 1000
	real(dp),parameter :: adj = 0.2
	real(dp),parameter :: tol_cap=1.0e-003
	real(dp),parameter :: tol_lab=1.0e-003
	real(dp),parameter :: tol_gy=1.0e-003

	!****** policy functions ******
	type(policy_function) :: policy_ini
	type(policy_egm) :: egm_ini

	!****** exogenous macro variables ******
	real(dp) :: tfp_gr
	real(dp) :: hmax
	real(dp) :: theta
	real(dp) :: delta
	real(dp) :: tax_cap
	real(dp) :: goy

	!****** endogenous macro variables ******
	real(dp) :: K_agg,K_supply
	real(dp) :: H_agg,H_supply
	real(dp) :: KoverH,KoverY
	real(dp) :: Y_agg

	!****** factor prices ******
	real(dp) :: wage,rent

	!****** tax rates ******
	real(dp) :: tax_ss
	real(dp) :: tax_lab
	real(dp) :: gov_exp,gov_exp_new

	!****** accidental bequest ******
	real(dp) :: beq
	
	!****** age-capital profile ******
	real(dp),dimension(js:jf) :: K_age,K_mean_age,K_next_age

	!****** age-labor profile ******
	real(dp),dimension(js:jr) :: H_age,H_mean_age,H_hour_age
	!------------------------------
	
	write(*,*) "Now, computing initial steady state"

	!/* set initial exogenous variables
	tfp_gr = state_ini%tfp_gr
	hmax = state_ini%hmax
	theta = state_ini%theta
	delta = state_ini%delta
	tax_cap = state_ini%tax_cap
	goy = state_ini%goy

	!/* initial guess of capital and labor
	K_agg = 1500.0
	H_agg = 500.0
	beq = 10.0
	KoverH = K_agg/H_agg

	!/* initial factor prices & payroll tax rate
	call factor_price(KoverH,tfp,theta,delta,rent,wage,KoverY)
	call govt_budget_ss(frac_ret(ts),tax_ss)

	!/* government expenditure
	Y_agg = tfp*(K_agg**theta)*(H_agg**(1.0-theta))
	gov_exp = goy*Y_agg
	tax_lab = 0.10

	!****** main loop ******
	do it_gy = 1,maxit

		if (metric_gy<tol_gy) exit
		metric_cap = 1.0
		metric_lab = 1.0

		do it_kl = 1,maxit

			if (metric_cap<tol_cap .and. metric_lab<tol_lab) exit

			!/* household's optimizaiton
			call end_grid_method_ss(ts,rent,wage,tax_ss,tax_lab,H_agg,beq,tfp_gr,hmax,theta,delta,tax_cap,policy_ini,egm_ini)

			!/* stationary distribution and profiles
			call law_of_motion_ss(policy_ini,phi_ini,hmax,K_mean_age,K_next_age,H_mean_age,H_hour_age)

			!/* aggregation
			do i = js,jf
				K_age(i) = mu(i,ts)*K_mean_age(i)
			end do
			do i = js,jr
				H_age(i) = mu(i,ts)*H_mean_age(i)
			end do
			K_supply = sum(K_age)
			H_supply = sum(H_age)

			!/* accidental bequest
			beq = 0.0
			do i = js,jf
				beq = (1.0-surv(i,ts))*mu(i,ts)*K_next_age(i) + beq
			end do

			!/* iteration error
			metric_cap = abs((K_supply-K_agg)/K_supply)
			metric_lab = abs((H_supply-H_agg)/H_supply)

			!/* up-data & new factor prices
			K_agg = adj*K_supply + (1.0-adj)*K_agg
			H_agg = adj*H_supply + (1.0-adj)*H_agg
			KoverH = K_agg/H_agg
			call factor_price(KoverH,tfp,theta,delta,rent,wage,KoverY)

		end do

		Y_agg = tfp*(K_agg**theta)*(H_agg**(1.0-theta))
		gov_exp_new = goy*Y_agg
		call govt_budget_general(gov_exp_new,rent,wage,K_agg,H_agg,tax_cap,tax_lab)
		metric_gy = abs((gov_exp-gov_exp_new)/gov_exp)
		gov_exp = gov_exp_new

	end do

	!/* results */
	write(*,"(' initial steady state: rent', f10.4,'%')") (1.0-tax_cap)*rent*100.0
	write(*,"(' initial steady state: wage', f10.4)") wage
	write(*,"(' initial steady state: K   ', f10.4)") K_agg
	write(*,"(' initial steady state: H   ', f10.4)") H_agg
	write(*,*) ""
	state_ini%rent = rent
	state_ini%wage = wage
	state_ini%cap = K_agg
	state_ini%lab = H_agg
	state_ini%beq = beq
	state_ini%gov_exp = gov_exp
	state_ini%tax_ss = tax_ss
	state_ini%tax_lab = tax_lab

end subroutine steady_state_ini